"""
Dictionaries module for Montreal function to hub function
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import hub.helpers.constants as cte


class MontrealFunctionToHubFunction:
  """
  Montreal function to hub function class
  """

  def __init__(self):
    self._dictionary = {'1000': cte.RESIDENTIAL,
                        '2089': cte.INDUSTRY,
                        '1921': cte.WAREHOUSE,
                        '1922': cte.NON_HEATED,
                        '9100': cte.NON_HEATED,
                        '6000': cte.MEDIUM_OFFICE,
                        '5010': cte.STAND_ALONE_RETAIL,
                        '9490': cte.WAREHOUSE,
                        '4299': cte.WAREHOUSE,
                        '6379': cte.WAREHOUSE,
                        '5533': cte.WAREHOUSE,
                        '6591': cte.OFFICE_AND_ADMINISTRATION,
                        '6211': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '1511': cte.DORMITORY,
                        '5833': cte.HOTEL,
                        '1532': cte.DORMITORY,
                        '6911': cte.CONVENTION_CENTER,
                        '9510': cte.RESIDENTIAL,
                        '1990': cte.MID_RISE_APARTMENT,
                        '1923': cte.NON_HEATED,
                        '7222': cte.SPORTS_LOCATION,
                        '5002': cte.STRIP_MALL,
                        '6111': cte.COMMERCIAL,
                        '6311': cte.MEDIUM_OFFICE,
                        '6399': cte.MEDIUM_OFFICE,
                        '5812': cte.FULL_SERVICE_RESTAURANT,
                        '4621': cte.WAREHOUSE,
                        '1541': cte.DORMITORY,
                        '7214': cte.EVENT_LOCATION,
                        '4821': cte.NON_HEATED,
                        '9520': cte.NON_HEATED,
                        '7112': cte.EVENT_LOCATION,
                        '6299': cte.OUT_PATIENT_HEALTH_CARE,
                        '5461': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '4632': cte.NON_HEATED,
                        '7424': cte.EVENT_LOCATION,
                        '5811': cte.FULL_SERVICE_RESTAURANT,
                        '4113': cte.WAREHOUSE,
                        '6821': cte.SECONDARY_SCHOOL,
                        '6920': cte.OFFICE_AND_ADMINISTRATION,
                        '6199': cte.COMMERCIAL,
                        '5899': cte.WAREHOUSE,
                        '5999': cte.STAND_ALONE_RETAIL,
                        '5834': cte.RESIDENTIAL,
                        '2699': cte.INDUSTRY,
                        '6812': cte.SECONDARY_SCHOOL,
                        '6649': cte.WAREHOUSE,
                        '3999': cte.INDUSTRY,
                        '1553': cte.OFFICE_AND_ADMINISTRATION,
                        '6999': cte.WAREHOUSE,
                        '6541': cte.PRIMARY_SCHOOL,
                        '5831': cte.SMALL_HOTEL,
                        '6919': cte.OFFICE_AND_ADMINISTRATION,
                        '9900': cte.WAREHOUSE,
                        '1551': cte.EVENT_LOCATION,
                        '5511': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6231': cte.COMMERCIAL,
                        '6221': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6599': cte.OFFICE_AND_ADMINISTRATION,
                        '7119': cte.EVENT_LOCATION,
                        '6214': cte.COMMERCIAL,
                        '5412': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '4839': cte.WAREHOUSE,
                        '6994': cte.COMMERCIAL,
                        '6344': cte.WAREHOUSE,
                        '6722': cte.WAREHOUSE,
                        '5111': cte.WAREHOUSE,
                        '6511': cte.OUT_PATIENT_HEALTH_CARE,
                        '5965': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4631': cte.WAREHOUSE,
                        '7451': cte.SPORTS_LOCATION,
                        '1539': cte.DORMITORY,
                        '6376': cte.WAREHOUSE,
                        '4633': cte.NON_HEATED,
                        '5813': cte.QUICK_SERVICE_RESTAURANT,
                        '6339': cte.COMMERCIAL,
                        '5911': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5651': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5971': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4550': cte.NON_HEATED,
                        '5620': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7611': cte.NON_HEATED,
                        '5531': cte.WAREHOUSE,
                        '6629': cte.WAREHOUSE,
                        '6521': cte.MEDIUM_OFFICE,
                        '7639': cte.WAREHOUSE,
                        '3399': cte.INDUSTRY,
                        '3019': cte.WAREHOUSE,
                        '6551': cte.MEDIUM_OFFICE,
                        '5413': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5821': cte.QUICK_SERVICE_RESTAURANT,
                        '6411': cte.WAREHOUSE,
                        '6799': cte.OFFICE_AND_ADMINISTRATION,
                        '5942': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7111': cte.OFFICE_AND_ADMINISTRATION,
                        '6419': cte.WAREHOUSE,
                        '6534': cte.COMMERCIAL,
                        '5020': cte.WAREHOUSE,
                        '6594': cte.MEDIUM_OFFICE,
                        '5819': cte.STAND_ALONE_RETAIL,
                        '6823': cte.SECONDARY_SCHOOL,
                        '4990': cte.WAREHOUSE,
                        '6759': cte.WAREHOUSE,
                        '6517': cte.OUT_PATIENT_HEALTH_CARE,
                        '6839': cte.SECONDARY_SCHOOL,
                        '4711': cte.MEDIUM_OFFICE,
                        '4111': cte.WAREHOUSE,
                        '3280': cte.WAREHOUSE,
                        '6721': cte.OFFICE_AND_ADMINISTRATION,
                        '9459': cte.NON_HEATED,
                        '7113': cte.EVENT_LOCATION,
                        '4743': cte.MEDIUM_OFFICE,
                        '7211': cte.EVENT_LOCATION,
                        '6513': cte.HOSPITAL,
                        '6813': cte.SECONDARY_SCHOOL,
                        '5921': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6383': cte.COMMERCIAL,
                        '5711': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6512': cte.OUT_PATIENT_HEALTH_CARE,
                        '1590': cte.OFFICE_AND_ADMINISTRATION,
                        '1890': cte.DORMITORY,
                        '6372': cte.WAREHOUSE,
                        '1600': cte.SMALL_HOTEL,
                        '9451': cte.NON_HEATED,
                        '6359': cte.WAREHOUSE,
                        '2240': cte.INDUSTRY,
                        '1559': cte.OFFICE_AND_ADMINISTRATION,
                        '1543': cte.DORMITORY,
                        '6760': cte.MEDIUM_OFFICE,
                        '4771': cte.MOTION_PICTURE_THEATRE,
                        '1552': cte.DORMITORY,
                        '3994': cte.INDUSTRY,
                        '6243': cte.NON_HEATED,
                        '6835': cte.SECONDARY_SCHOOL,
                        '5004': cte.STRIP_MALL,
                        '6791': cte.OFFICE_AND_ADMINISTRATION,
                        '1510': cte.DORMITORY,
                        '7239': cte.OFFICE_AND_ADMINISTRATION,
                        '5492': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5499': cte.STAND_ALONE_RETAIL,
                        '4413': cte.WAREHOUSE,
                        '5951': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5411': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '5532': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5252': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6395': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5462': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '6232': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6641': cte.WAREHOUSE,
                        '7432': cte.SPORTS_LOCATION,
                        '5431': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '6532': cte.OFFICE_AND_ADMINISTRATION,
                        '6518': cte.OUT_PATIENT_HEALTH_CARE,
                        '5199': cte.WAREHOUSE,
                        '3840': cte.INDUSTRY,
                        '2051': cte.MEDIUM_OFFICE,
                        '7290': cte.OFFICE_AND_ADMINISTRATION,
                        '5253': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5660': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5699': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6579': cte.OUT_PATIENT_HEALTH_CARE,
                        '5173': cte.WAREHOUSE,
                        '6431': cte.WAREHOUSE,
                        '6355': cte.WAREHOUSE,
                        '6263': cte.OUT_PATIENT_HEALTH_CARE,
                        '5610': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5969': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '5991': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7219': cte.OFFICE_AND_ADMINISTRATION,
                        '7117': cte.WAREHOUSE,
                        '5005': cte.STAND_ALONE_RETAIL,
                        '4561': cte.NON_HEATED,
                        '5631': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4623': cte.WAREHOUSE,
                        '4590': cte.WAREHOUSE,
                        '3899': cte.INDUSTRY,
                        '6563': cte.COMMERCIAL,
                        '5399': cte.STAND_ALONE_RETAIL,
                        '6241': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6351': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '2270': cte.INDUSTRY,
                        '5132': cte.WAREHOUSE,
                        '4221': cte.WAREHOUSE,
                        '6253': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5640': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3299': cte.INDUSTRY,
                        '6831': cte.SECONDARY_SCHOOL,
                        '2012': cte.INDUSTRY,
                        '3620': cte.INDUSTRY,
                        '6519': cte.OUT_PATIENT_HEALTH_CARE,
                        '2291': cte.INDUSTRY,
                        '5311': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5931': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4611': cte.WAREHOUSE,
                        '4719': cte.STAND_ALONE_RETAIL,
                        '5693': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4490': cte.INDUSTRY,
                        '5823': cte.FULL_SERVICE_RESTAURANT,
                        '2045': cte.INDUSTRY,
                        '6542': cte.HOSPITAL,
                        '2014': cte.INDUSTRY,
                        '6375': cte.WAREHOUSE,
                        '2799': cte.INDUSTRY,
                        '6612': cte.WAREHOUSE,
                        '4222': cte.WAREHOUSE,
                        '7191': cte.EVENT_LOCATION,
                        '6352': cte.WAREHOUSE,
                        '5836': cte.COMMERCIAL,
                        '5952': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '1512': cte.DORMITORY,
                        '5948': cte.WAREHOUSE,
                        '3580': cte.INDUSTRY,
                        '5391': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6997': cte.OFFICE_AND_ADMINISTRATION,
                        '6539': cte.OFFICE_AND_ADMINISTRATION,
                        '2899': cte.INDUSTRY,
                        '1100': cte.SINGLE_FAMILY_HOUSE,
                        '6348': cte.WAREHOUSE,
                        '7499': cte.EVENT_LOCATION,
                        '6373': cte.WAREHOUSE,
                        '3539': cte.INDUSTRY,
                        '4922': cte.WAREHOUSE,
                        '6731': cte.OFFICE_AND_ADMINISTRATION,
                        '6132': cte.MEDIUM_OFFICE,
                        '2073': cte.INDUSTRY,
                        '4761': cte.COMMERCIAL,
                        '7312': cte.NON_HEATED,
                        '7425': cte.SPORTS_LOCATION,
                        '6634': cte.INDUSTRY,
                        '6639': cte.WAREHOUSE,
                        '3231': cte.INDUSTRY,
                        '6569': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '2499': cte.INDUSTRY,
                        '7199': cte.EVENT_LOCATION,
                        '7114': cte.EVENT_LOCATION,
                        '4733': cte.COMMERCIAL,
                        '4112': cte.WAREHOUSE,
                        '7129': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7620': cte.NON_HEATED,
                        '9390': cte.WAREHOUSE,
                        '1529': cte.OFFICE_AND_ADMINISTRATION,
                        '7417': cte.NON_HEATED,
                        '8221': cte.OUT_PATIENT_HEALTH_CARE,
                        '6598': cte.OUT_PATIENT_HEALTH_CARE,
                        '6515': cte.OUT_PATIENT_HEALTH_CARE,
                        '3719': cte.WAREHOUSE,
                        '2471': cte.INDUSTRY,
                        '7433': cte.NON_HEATED,
                        '6413': cte.WAREHOUSE,
                        '3459': cte.INDUSTRY,
                        '6152': cte.MEDIUM_OFFICE,
                        '2093': cte.INDUSTRY,
                        '3031': cte.INDUSTRY,
                        '5251': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5394': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5141': cte.STRIP_MALL,
                        '7419': cte.SPORTS_LOCATION,
                        '4293': cte.WAREHOUSE,
                        '6834': cte.SECONDARY_SCHOOL,
                        '2092': cte.INDUSTRY,
                        '2072': cte.INDUSTRY,
                        '6712': cte.MEDIUM_OFFICE,
                        '4411': cte.WAREHOUSE,
                        '4562': cte.NON_HEATED,
                        '2096': cte.INDUSTRY,
                        '5997': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7212': cte.EVENT_LOCATION,
                        '6713': cte.MEDIUM_OFFICE,
                        '9212': cte.NON_HEATED,
                        '6725': cte.OFFICE_AND_ADMINISTRATION,
                        '9410': cte.RESIDENTIAL,
                        '4211': cte.WAREHOUSE,
                        '5230': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6141': cte.MEDIUM_OFFICE,
                        '7512': cte.HEALTH_CARE,
                        '5822': cte.QUICK_SERVICE_RESTAURANT,
                        '5121': cte.STRIP_MALL,
                        '6369': cte.SECONDARY_SCHOOL,
                        '5171': cte.STRIP_MALL,
                        '5003': cte.STRIP_MALL,
                        '3630': cte.INDUSTRY,
                        '5521': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6531': cte.OUT_PATIENT_HEALTH_CARE,
                        '7413': cte.WAREHOUSE,
                        '3239': cte.INDUSTRY,
                        '4715': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6365': cte.UNIVERSITY,
                        '6121': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6611': cte.WAREHOUSE,
                        '3649': cte.INDUSTRY,
                        '2892': cte.INDUSTRY,
                        '6269': cte.OUT_PATIENT_HEALTH_CARE,
                        '2020': cte.INDUSTRY,
                        '6621': cte.WAREHOUSE,
                        '6312': cte.NON_HEATED,
                        '6416': cte.INDUSTRY,
                        '5712': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2261': cte.INDUSTRY,
                        '2931': cte.INDUSTRY,
                        '3011': cte.INDUSTRY,
                        '6499': cte.STAND_ALONE_RETAIL,
                        '3599': cte.INDUSTRY,
                        '2299': cte.INDUSTRY,
                        '3831': cte.INDUSTRY,
                        '2410': cte.INDUSTRY,
                        '5112': cte.STRIP_MALL,
                        '5941': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6822': cte.SECONDARY_SCHOOL,
                        '6753': cte.WAREHOUSE,
                        '5149': cte.STRIP_MALL,
                        '3190': cte.INDUSTRY,
                        '6414': cte.WAREHOUSE,
                        '6633': cte.WAREHOUSE,
                        '3895': cte.INDUSTRY,
                        '5133': cte.STRIP_MALL,
                        '2082': cte.INDUSTRY,
                        '5512': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '9420': cte.NON_HEATED,
                        '6543': cte.HOSPITAL,
                        '4841': cte.INDUSTRY,
                        '4851': cte.INDUSTRY,
                        '5432': cte.STRIP_MALL,
                        '3711': cte.INDUSTRY,
                        '3460': cte.INDUSTRY,
                        '2087': cte.INDUSTRY,
                        '1522': cte.HALL,
                        '8549': cte.INDUSTRY,
                        '6242': cte.WAREHOUSE,
                        '6412': cte.WAREHOUSE,
                        '6811': cte.SECONDARY_SCHOOL,
                        '6154': cte.WAREHOUSE,
                        '7123': cte.WAREHOUSE,
                        '6993': cte.MEDIUM_OFFICE,
                        '6742': cte.HOSPITAL,
                        '5212': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5211': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '7990': cte.HALL,
                        '3714': cte.INDUSTRY,
                        '6593': cte.SECONDARY_SCHOOL,
                        '5172': cte.STRIP_MALL,
                        '5955': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6635': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3799': cte.INDUSTRY,
                        '2819': cte.INDUSTRY,
                        '5894': cte.QUICK_SERVICE_RESTAURANT,
                        '9530': cte.NON_HEATED,
                        '3159': cte.INDUSTRY,
                        '3713': cte.WAREHOUSE,
                        '3894': cte.INDUSTRY,
                        '5721': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5169': cte.STRIP_MALL,
                        '5593': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6631': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5731': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2739': cte.INDUSTRY,
                        '6619': cte.WAREHOUSE,
                        '6815': cte.SECONDARY_SCHOOL,
                        '5320': cte.STRIP_MALL,
                        '3261': cte.INDUSTRY,
                        '6394': cte.WAREHOUSE,
                        '2031': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '6423': cte.WAREHOUSE,
                        '3162': cte.INDUSTRY,
                        '5814': cte.QUICK_SERVICE_RESTAURANT,
                        '6653': cte.WAREHOUSE,
                        '2213': cte.INDUSTRY,
                        '2046': cte.INDUSTRY,
                        '6251': cte.COMMERCIAL,
                        '3650': cte.INDUSTRY,
                        '4799': cte.NON_HEATED,
                        '5832': cte.SMALL_HOTEL,
                        '4229': cte.WAREHOUSE,
                        '4842': cte.WAREHOUSE,
                        '5163': cte.STRIP_MALL,
                        '5148': cte.STRIP_MALL,
                        '2011': cte.INDUSTRY,
                        '5361': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2999': cte.INDUSTRY,
                        '6522': cte.MEDIUM_OFFICE,
                        '7121': cte.MUSEUM,
                        '7221': cte.SPORTS_LOCATION,
                        '1549': cte.OFFICE_AND_ADMINISTRATION,
                        '5652': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3331': cte.INDUSTRY,
                        '6219': cte.WAREHOUSE,
                        '5421': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '3861': cte.INDUSTRY,
                        '3919': cte.INDUSTRY,
                        '6441': cte.WAREHOUSE,
                        '6648': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '8199': cte.INDUSTRY,
                        '6832': cte.SECONDARY_SCHOOL,
                        '6992': cte.MEDIUM_OFFICE,
                        '1521': cte.EVENT_LOCATION,
                        '6349': cte.WAREHOUSE,
                        '4921': cte.WAREHOUSE,
                        '6498': cte.WAREHOUSE,
                        '6415': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4122': cte.WAREHOUSE,
                        '4219': cte.WAREHOUSE,
                        '9440': cte.NON_HEATED,
                        '4782': cte.DATACENTER,
                        '2619': cte.INDUSTRY,
                        '7229': cte.SPORTS_LOCATION,
                        '2079': cte.INDUSTRY,
                        '2320': cte.INDUSTRY,
                        '3259': cte.INDUSTRY,
                        '3931': cte.INDUSTRY,
                        '5470': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '4890': cte.WAREHOUSE,
                        '5599': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3270': cte.INDUSTRY,
                        '5186': cte.STRIP_MALL,
                        '2043': cte.INDUSTRY,
                        '6733': cte.WAREHOUSE,
                        '4311': cte.EVENT_LOCATION,
                        '3451': cte.INDUSTRY,
                        '2812': cte.INDUSTRY,
                        '2130': cte.INDUSTRY,
                        '7449': cte.WAREHOUSE,
                        '2919': cte.INDUSTRY,
                        '4315': cte.WAREHOUSE,
                        '3569': cte.INDUSTRY,
                        '7399': cte.EVENT_LOCATION,
                        '6160': cte.OFFICE_AND_ADMINISTRATION,
                        '7412': cte.NON_HEATED,
                        '2084': cte.INDUSTRY,
                        '3870': cte.INDUSTRY,
                        '5001': cte.STRIP_MALL,
                        '3411': cte.INDUSTRY,
                        '1702': cte.NON_HEATED,
                        '3243': cte.INDUSTRY,
                        '4926': cte.MEDIUM_OFFICE,
                        '3490': cte.INDUSTRY,
                        '2219': cte.INDUSTRY,
                        '2829': cte.INDUSTRY,
                        '4399': cte.WAREHOUSE,
                        '4319': cte.WAREHOUSE,
                        '2039': cte.INDUSTRY,
                        '8139': cte.GREEN_HOUSE,
                        '5146': cte.STRIP_MALL,
                        '3396': cte.INDUSTRY,
                        '5161': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2091': cte.INDUSTRY,
                        '3850': cte.INDUSTRY,
                        '5241': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3049': cte.INDUSTRY,
                        '5181': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2627': cte.INDUSTRY,
                        '3412': cte.INDUSTRY,
                        '5220': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3699': cte.INDUSTRY,
                        '4929': cte.WAREHOUSE,
                        '3551': cte.INDUSTRY,
                        '5198': cte.STRIP_MALL,
                        '6646': cte.NON_HEATED,
                        '5189': cte.STRIP_MALL,
                        '5312': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4214': cte.WAREHOUSE,
                        '3410': cte.INDUSTRY,
                        '4859': cte.WAREHOUSE,
                        '2736': cte.INDUSTRY,
                        '6642': cte.WAREHOUSE,
                        '3921': cte.INDUSTRY,
                        '5815': cte.FULL_SERVICE_RESTAURANT,
                        '6514': cte.OUT_PATIENT_HEALTH_CARE,
                        '8399': cte.WAREHOUSE,
                        '2250': cte.INDUSTRY,
                        '6378': cte.WAREHOUSE,
                        '6343': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2652': cte.INDUSTRY,
                        '3552': cte.INDUSTRY,
                        '2891': cte.INDUSTRY,
                        '6368': cte.SECONDARY_SCHOOL,
                        '3559': cte.INDUSTRY,
                        '5145': cte.STRIP_MALL,
                        '3244': cte.INDUSTRY,
                        '3292': cte.INDUSTRY,
                        '4510': cte.WAREHOUSE,
                        '7423': cte.SPORTS_LOCATION,
                        '5370': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4824': cte.WAREHOUSE,
                        '4832': cte.INDUSTRY,
                        '6573': cte.HEALTH_CARE,
                        '6623': cte.NON_HEATED,
                        '5162': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4612': cte.WAREHOUSE,
                        '2629': cte.INDUSTRY,
                        '3291': cte.INDUSTRY,
                        '3229': cte.INDUSTRY,
                        '5829': cte.STAND_ALONE_RETAIL,
                        '2932': cte.INDUSTRY,
                        '5594': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2722': cte.INDUSTRY,
                        '2811': cte.INDUSTRY,
                        '2235': cte.INDUSTRY,
                        '5953': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '3170': cte.INDUSTRY,
                        '3662': cte.INDUSTRY,
                        '4879': cte.WAREHOUSE,
                        '2074': cte.INDUSTRY,
                        '7429': cte.SPORTS_LOCATION,
                        '4843': cte.WAREHOUSE,
                        '6335': cte.OFFICE_AND_ADMINISTRATION,
                        '4116': cte.WAREHOUSE,
                        '2622': cte.INDUSTRY,
                        '6112': cte.OFFICE_AND_ADMINISTRATION,
                        '4875': cte.WAREHOUSE,
                        '4792': cte.WAREHOUSE_REFRIGERATED,
                        '6391': cte.UNIVERSITY,
                        '6425': cte.WAREHOUSE,
                        '6212': cte.WAREHOUSE,
                        '5147': cte.STRIP_MALL,
                        '2460': cte.INDUSTRY,
                        '4874': cte.WAREHOUSE,
                        '6743': cte.DORMITORY,
                        '3241': cte.INDUSTRY,
                        '5123': cte.STRIP_MALL,
                        '4833': cte.NON_HEATED,
                        '9470': cte.NON_HEATED,
                        '3932': cte.INDUSTRY,
                        '4292': cte.WAREHOUSE,
                        '6424': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '6139': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '2624': cte.INDUSTRY,
                        '3915': cte.WAREHOUSE,
                        '7233': cte.CONVENTION_CENTER,
                        '5184': cte.STRIP_MALL,
                        '2099': cte.INDUSTRY,
                        '5129': cte.RETAIL_SHOP_WITH_REFRIGERATED_FOOD,
                        '5913': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '5717': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4712': cte.NON_HEATED,
                        '5142': cte.STRIP_MALL,
                        '3210': cte.INDUSTRY,
                        '4834': cte.WAREHOUSE,
                        '3456': cte.INDUSTRY,
                        '2075': cte.INDUSTRY,
                        '3020': cte.INDUSTRY,
                        '3821': cte.INDUSTRY,
                        '6613': cte.MEDIUM_OFFICE,
                        '4826': cte.WAREHOUSE,
                        '5839': cte.MULTI_FAMILY_HOUSE,
                        '4316': cte.WAREHOUSE,
                        '6592': cte.OFFICE_AND_ADMINISTRATION,
                        '3971': cte.INDUSTRY,
                        '3972': cte.INDUSTRY,
                        '2694': cte.INDUSTRY,
                        '3882': cte.INDUSTRY,
                        '3119': cte.INDUSTRY,
                        '2495': cte.INDUSTRY,
                        '4759': cte.MEDIUM_OFFICE,
                        '6439': cte.WAREHOUSE,
                        '6643': cte.WAREHOUSE,
                        '3470': cte.INDUSTRY,
                        '3531': cte.INDUSTRY,
                        '4823': cte.NON_HEATED,
                        '2293': cte.INDUSTRY,
                        '3532': cte.INDUSTRY,
                        '3913': cte.INDUSTRY,
                        '6816': cte.OFFICE_AND_ADMINISTRATION,
                        '3562': cte.INDUSTRY,
                        '6496': cte.COMMERCIAL,
                        '4119': cte.WAREHOUSE,
                        '6533': cte.OUT_PATIENT_HEALTH_CARE,
                        '6814': cte.PRIMARY_SCHOOL,
                        '6353': cte.WAREHOUSE,
                        '3392': cte.INDUSTRY,
                        '5114': cte.STRIP_MALL,
                        '5131': cte.STRIP_MALL,
                        '3641': cte.INDUSTRY,
                        '2614': cte.INDUSTRY,
                        '3661': cte.INDUSTRY,
                        '2081': cte.INDUSTRY,
                        '3340': cte.INDUSTRY,
                        '4928': cte.WORKSHOP,
                        '3712': cte.INDUSTRY,
                        '3253': cte.INDUSTRY,
                        '3860': cte.INDUSTRY,
                        '3892': cte.INDUSTRY,
                        '2992': cte.INDUSTRY,
                        '5598': cte.RETAIL_SHOP_WITHOUT_REFRIGERATED_FOOD,
                        '4825': cte.NON_HEATED,
                        '2071': cte.INDUSTRY,
                        '6495': cte.INDUSTRY,
                        '2914': cte.INDUSTRY,
                        '3182': cte.INDUSTRY,
                        '3791': cte.INDUSTRY,
                        '3992': cte.INDUSTRY,
                        '3114': cte.INDUSTRY,
                        '7920': cte.OFFICE_AND_ADMINISTRATION,
                        '5891': cte.RESTAURANT,
                        '5835': cte.SMALL_HOTEL,
                        '6565': cte.HEALTH_CARE,
                        '3391': cte.INDUSTRY,
                        '6615': cte.INDUSTRY,
                        '3883': cte.INDUSTRY,
                        '2032': cte.INDUSTRY,
                        '2994': cte.INDUSTRY,
                        '4871': cte.INDUSTRY,
                        '5113': cte.WORKSHOP,
                        '3571': cte.INDUSTRY,
                        '2342': cte.INDUSTRY,
                        '3911': cte.INDUSTRY,
                        '7444': cte.EDUCATION,
                        '2221': cte.INDUSTRY,
                        '8192': cte.FARM,
                        '2439': cte.INDUSTRY,
                        '3891': cte.INDUSTRY,
                        '6354': cte.WORKSHOP,
                        '4815': cte.NON_HEATED,
                        '6651': cte.WORKSHOP,
                        '2822': cte.INDUSTRY,
                        '2821': cte.INDUSTRY
                        }

  @property
  def dictionary(self) -> dict:
    """
    Get the dictionary
    :return: {}
    """
    return self._dictionary



"""
Palma usage catalog
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Cecilia Pérez cperez@irec.cat
"""

import json
import urllib.request
from pathlib import Path

import xmltodict

import hub.helpers.constants as cte
from hub.catalog_factories.catalog import Catalog
from hub.catalog_factories.data_models.usages.appliances import Appliances
from hub.catalog_factories.data_models.usages.content import Content
from hub.catalog_factories.data_models.usages.lighting import Lighting
from hub.catalog_factories.data_models.usages.occupancy import Occupancy
from hub.catalog_factories.data_models.usages.domestic_hot_water import DomesticHotWater
from hub.catalog_factories.data_models.usages.schedule import Schedule
from hub.catalog_factories.data_models.usages.thermal_control import ThermalControl
from hub.catalog_factories.data_models.usages.usage import Usage
from hub.catalog_factories.usage.usage_helper import UsageHelper


class PalmaCatalog(Catalog):
  """
  Palma catalog class
  """
  def __init__(self, path):
    self._schedules_path = Path(path / 'palma_schedules.json').resolve()
    self._space_types_path = Path(path / 'palma_space_types.json').resolve()
    self._space_compliance_path = Path(path / 'palma_space_compliance.json').resolve()
    self._content = None
    self._schedules = {}
    self._load_schedules()
    self._content = Content(self._load_archetypes())

  @staticmethod
  def _extract_schedule(raw):
    nrcan_schedule_type = raw['category']
    if 'Heating' in raw['name'] and 'Water' not in raw['name']:
      nrcan_schedule_type = f'{nrcan_schedule_type} Heating'
    elif 'Cooling' in raw['name']:
      nrcan_schedule_type = f'{nrcan_schedule_type} Cooling'
    if nrcan_schedule_type not in UsageHelper().nrcan_schedule_type_to_hub_schedule_type:
      return None
    hub_type = UsageHelper().nrcan_schedule_type_to_hub_schedule_type[nrcan_schedule_type]
    data_type = UsageHelper().nrcan_data_type_to_hub_data_type[raw['units']]
    time_step = UsageHelper().nrcan_time_to_hub_time[raw['type']]
    # nrcan only uses daily range for the schedules
    time_range = cte.DAY
    day_types = UsageHelper().nrcan_day_type_to_hub_days[raw['day_types']]
    return Schedule(hub_type, raw['values'], data_type, time_step, time_range, day_types)

  def _load_schedules(self):
    _schedule_types = []
    with open(self._schedules_path, 'r') as f:
      schedules_type = json.load(f)
    for schedule_type in schedules_type['tables']['schedules']['table']:
      schedule = PalmaCatalog._extract_schedule(schedule_type)
      if schedule_type['name'] not in _schedule_types:
        _schedule_types.append(schedule_type['name'])
        if schedule is not None:
          self._schedules[schedule_type['name']] = [schedule]
      else:
        if schedule is not None:
          _schedules = self._schedules[schedule_type['name']]
          _schedules.append(schedule)
          self._schedules[schedule_type['name']] = _schedules

  def _get_schedules(self, name):
    schedule = None
    if name in self._schedules:
      schedule = self._schedules[name]
    return schedule

  def _load_archetypes(self):
    usages = []
    with open(self._space_types_path, 'r') as f:
      space_types = json.load(f)['tables']['space_types']['table']
    space_types = [st for st in space_types if st['space_type'] == 'WholeBuilding']
    with open(self._space_compliance_path, 'r') as f:
      space_types_compliance = json.load(f)['tables']['space_compliance']['table']
    space_types_compliance = [st for st in space_types_compliance if st['space_type'] == 'WholeBuilding']
    space_types_dictionary = {}
    for space_type in space_types_compliance:
      usage_type = space_type['building_type']
      # people/m2
      occupancy_density = space_type['occupancy_per_area_people_per_m2']
      # W/m2
      lighting_density = space_type['lighting_per_area_w_per_m2']
      # W/m2
      appliances_density = space_type['electric_equipment_per_area_w_per_m2']
      # peak flow in gallons/h/m2
      domestic_hot_water_peak_flow = (
          space_type['service_water_heating_peak_flow_per_area'] *
          cte.GALLONS_TO_QUBIC_METERS / cte.HOUR_TO_SECONDS
      )
      space_types_dictionary[usage_type] = {'occupancy_per_area': occupancy_density,
                                            'lighting_per_area': lighting_density,
                                            'electric_equipment_per_area': appliances_density,
                                            'service_water_heating_peak_flow_per_area': domestic_hot_water_peak_flow
                                            }

    for space_type in space_types:
      usage_type = space_type['building_type']
      space_type_compliance = space_types_dictionary[usage_type]
      occupancy_density = space_type_compliance['occupancy_per_area']
      sensible_convective_internal_gain = space_type['sensible_convective_internal_gain']
      sensible_radiative_internal_gain = space_type['sensible_radiative_internal_gain']
      latent_internal_gain = space_type['latent_internal_gain']
      lighting_density = space_type_compliance['lighting_per_area']
      appliances_density = space_type_compliance['electric_equipment_per_area']
      domestic_hot_water_peak_flow = space_type_compliance['service_water_heating_peak_flow_per_area']

      occupancy_schedule_name = space_type['occupancy_schedule']
      lighting_schedule_name = space_type['lighting_schedule']
      appliance_schedule_name = space_type['electric_equipment_schedule']
      hvac_schedule_name = space_type['exhaust_schedule']
      if hvac_schedule_name and 'FAN' in hvac_schedule_name:
        hvac_schedule_name = hvac_schedule_name.replace('FAN', 'Fan')
      if not hvac_schedule_name:
        hvac_schedule_name = 'default_HVAC_schedule'
      heating_setpoint_schedule_name = space_type['heating_setpoint_schedule']
      cooling_setpoint_schedule_name = space_type['cooling_setpoint_schedule']
      domestic_hot_water_schedule_name = space_type['service_water_heating_schedule']
      occupancy_schedule = self._get_schedules(occupancy_schedule_name)
      lighting_schedule = self._get_schedules(lighting_schedule_name)
      appliance_schedule = self._get_schedules(appliance_schedule_name)
      heating_schedule = self._get_schedules(heating_setpoint_schedule_name)
      cooling_schedule = self._get_schedules(cooling_setpoint_schedule_name)
      hvac_availability = self._get_schedules(hvac_schedule_name)
      domestic_hot_water_load_schedule = self._get_schedules(domestic_hot_water_schedule_name)

      # ACH -> 1/s
      mechanical_air_change = space_type['ventilation_air_changes'] / cte.HOUR_TO_SECONDS
      # cfm/ft2 to m3/m2.s
      ventilation_rate = space_type['ventilation_per_area'] / (cte.METERS_TO_FEET * cte.MINUTES_TO_SECONDS)
      # cfm/person to m3/m2.s
      ventilation_rate += space_type['ventilation_per_person'] / (
          pow(cte.METERS_TO_FEET, 3) * cte.MINUTES_TO_SECONDS
      ) * occupancy_density

      lighting_radiative_fraction = space_type['lighting_fraction_radiant']
      lighting_convective_fraction = 0
      if lighting_radiative_fraction is not None:
        lighting_convective_fraction = 1 - lighting_radiative_fraction
      lighting_latent_fraction = 0
      appliances_radiative_fraction = space_type['electric_equipment_fraction_radiant']
      appliances_latent_fraction = space_type['electric_equipment_fraction_latent']
      appliances_convective_fraction = 0
      if appliances_radiative_fraction is not None and appliances_latent_fraction is not None:
        appliances_convective_fraction = 1 - appliances_radiative_fraction - appliances_latent_fraction

      domestic_hot_water_service_temperature = space_type['service_water_heating_target_temperature']

      occupancy = Occupancy(occupancy_density,
                            sensible_convective_internal_gain,
                            sensible_radiative_internal_gain,
                            latent_internal_gain,
                            occupancy_schedule)
      lighting = Lighting(lighting_density,
                          lighting_convective_fraction,
                          lighting_radiative_fraction,
                          lighting_latent_fraction,
                          lighting_schedule)
      appliances = Appliances(appliances_density,
                              appliances_convective_fraction,
                              appliances_radiative_fraction,
                              appliances_latent_fraction,
                              appliance_schedule)
      thermal_control = ThermalControl(None,
                                       None,
                                       None,
                                       hvac_availability,
                                       heating_schedule,
                                       cooling_schedule)
      domestic_hot_water = DomesticHotWater(None,
                                            domestic_hot_water_peak_flow,
                                            domestic_hot_water_service_temperature,
                                            domestic_hot_water_load_schedule)

      hours_day = None
      days_year = None

      usages.append(Usage(usage_type,
                          hours_day,
                          days_year,
                          mechanical_air_change,
                          ventilation_rate,
                          occupancy,
                          lighting,
                          appliances,
                          thermal_control,
                          domestic_hot_water))
    

    return usages

  def names(self, category=None):
    """
    Get the catalog elements names
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    _names = {'usages': []}
    for usage in self._content.usages:
      _names['usages'].append(usage.name)
    return _names

  def entries(self, category=None):
    """
    Get the catalog elements
    :parm: for usage catalog category filter does nothing as there is only one category (usages)
    """
    return self._content

  def get_entry(self, name):
    """
    Get one catalog element by names
    :parm: entry name
    """
    for usage in self._content.usages:
      if usage.name.lower() == name.lower():
        return usage
    raise IndexError(f"{name} doesn't exists in the catalog")


"""
Dictionaries module saves all transformations of functions and usages to access the catalogs
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.helpers.parsers.list_usage_to_hub import ListUsageToHub
from hub.helpers.parsers.string_usage_to_hub import StringUsageToHub 

class UsageParsers:
  """
  Dictionaries class
  """

  @staticmethod
  def string_usage_to_hub() -> object:
    """
    Hub usage to HfT usage, transformation dictionary
    :return: dict
    """
    return StringUsageToHub().parse

  @staticmethod
  def list_usage_to_hub(function_dictionary=None) -> object:
    """
    Hub usage to HfT usage, transformation dictionary
    :return: dict
    """
    return ListUsageToHub(function_dictionary).parse




"""
ExportsFactory export a city into several formats
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""

from pathlib import Path

from hub.exports.formats.glb import Glb
from hub.exports.formats.obj import Obj
from hub.exports.formats.geojson import Geojson
from hub.exports.formats.simplified_radiosity_algorithm import SimplifiedRadiosityAlgorithm
from hub.exports.formats.stl import Stl
from hub.exports.formats.cesiumjs_tileset import CesiumjsTileset
from hub.helpers.utils import validate_import_export_type


class ExportsFactory:
  """
  Exports factory class
  """
  def __init__(self, handler, city, path, target_buildings=None, adjacent_buildings=None, base_uri=None):
    self._city = city
    self._handler = '_' + handler.lower()
    validate_import_export_type(ExportsFactory, handler)
    if isinstance(path, str):
      path = Path(path)
    self._path = path
    self._target_buildings = target_buildings
    self._adjacent_buildings = adjacent_buildings
    self._base_uri = base_uri

  @property
  def _stl(self):
    """
    Export the city geometry to stl
    :return: None
    """
    return Stl(self._city, self._path)

  @property
  def _obj(self):
    """
    Export the city geometry to obj
    :return: None
    """
    return Obj(self._city, self._path)

  @property
  def _sra(self):
    """
    Export the city to Simplified Radiosity Algorithm xml format
    :return: None
    """
    return SimplifiedRadiosityAlgorithm(
      self._city, (self._path / f'{self._city.name}_sra.xml'), target_buildings=self._target_buildings
    )

  @property
  def _cesiumjs_tileset(self):
    """
    Export the city to a cesiumJs tileset format
    :return: None
    """
    return CesiumjsTileset(
      self._city,
      (self._path / f'{self._city.name}.json'),
      target_buildings=self._target_buildings,
      base_uri=self._base_uri
    )

  @property
  def _glb(self):
    return Glb(self._city, self._path, target_buildings=self._target_buildings)

  @property
  def _geojson(self):
    return Geojson(self._city, self._path, target_buildings=self._target_buildings)

  def export(self):
    """
    Export the city given to the class using the given export type handler
    :return: None
    """
    return getattr(self, self._handler, lambda: None)



"""
Energy system definition
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from typing import Union, List
from pathlib import Path

from hub.city_model_structure.energy_systems.distribution_system import DistributionSystem
from hub.city_model_structure.energy_systems.non_pv_generation_system import NonPvGenerationSystem
from hub.city_model_structure.energy_systems.pv_generation_system import PvGenerationSystem
from hub.city_model_structure.energy_systems.control_system import ControlSystem
from hub.city_model_structure.city_object import CityObject


class EnergySystem:
  """
  EnergySystem class
  """
  def __init__(self):
    self._demand_types = None
    self._name = None
    self._generation_systems = None
    self._distribution_systems = None
    self._configuration_schema = None
    self._connected_city_objects = None
    self._control_system = None

  @property
  def demand_types(self):
    """
    Get demand able to cover from [Heating, Cooling, Domestic Hot Water, Electricity]
    :return: [string]
    """
    return self._demand_types

  @demand_types.setter
  def demand_types(self, value):
    """
    Set demand able to cover from [Heating, Cooling, Domestic Hot Water, Electricity]
    :param value: [string]
    """
    self._demand_types = value

  @property
  def name(self):
    """
    Get energy system name
    :return: str
    """
    return self._name

  @name.setter
  def name(self, value):
    """
    Set energy system name
    :param value:
    """
    self._name = value

  @property
  def generation_systems(self) -> Union[List[NonPvGenerationSystem], List[PvGenerationSystem]]:
    """
    Get generation systems
    :return: [GenerationSystem]
    """
    return self._generation_systems

  @generation_systems.setter
  def generation_systems(self, value):
    """
    Set generation systems
    :return: [GenerationSystem]
    """
    self._generation_systems = value

  @property
  def distribution_systems(self) -> Union[None, List[DistributionSystem]]:
    """
    Get distribution systems
    :return: [DistributionSystem]
    """
    return self._distribution_systems

  @distribution_systems.setter
  def distribution_systems(self, value):
    """
    Set distribution systems
    :param value: [DistributionSystem]
    """
    self._distribution_systems = value

  @property
  def configuration_schema(self) -> Path:
    """
    Get the schema of the system configuration
    :return: Path
    """
    return self._configuration_schema

  @configuration_schema.setter
  def configuration_schema(self, value):
    """
    Set the schema of the system configuration
    :param value: Path
    """
    self._configuration_schema = value

  @property
  def connected_city_objects(self) -> Union[None, List[CityObject]]:
    """
    Get list of city objects that are connected to this energy system
    :return: [CityObject]
    """
    return self._connected_city_objects

  @connected_city_objects.setter
  def connected_city_objects(self, value):
    """
    Set list of city objects that are connected to this energy system
    :param value: [CityObject]
    """
    self._connected_city_objects = value

  @property
  def control_system(self) -> Union[None, ControlSystem]:
    """
    Get control system of the energy system
    :return: ControlSystem
    """
    return self._control_system

  @control_system.setter
  def control_system(self, value):
    """
    Set control system of the energy system
    :param value: ControlSystem
    """
    self._control_system = value



"""
Thermal archetype module
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.city_model_structure.building_demand.construction import Construction


class ThermalArchetype:
  """
  ThermalArchetype class
  """
  def __init__(self):
    self._constructions = None
    self._average_storey_height = None
    self._thermal_capacity = None
    self._extra_loses_due_to_thermal_bridges = None
    self._indirect_heated_ratio = None
    self._infiltration_rate_for_ventilation_system_off = None
    self._infiltration_rate_for_ventilation_system_on = None
    self._infiltration_rate_area_for_ventilation_system_off=None
    self._infiltration_rate_area_for_ventilation_system_on=None

  @property
  def constructions(self) -> [Construction]:
    """
    Get archetype constructions
    :return: [Construction]
    """
    return self._constructions

  @constructions.setter
  def constructions(self, value):
    """
    Set archetype constructions
    :param value: [Construction]
    """
    self._constructions = value

  @property
  def average_storey_height(self):
    """
    Get average storey height in m
    :return: float
    """
    return self._average_storey_height

  @average_storey_height.setter
  def average_storey_height(self, value):
    """
    Set average storey height in m
    :param value: float
    """
    self._average_storey_height = value

  @property
  def thermal_capacity(self):
    """
    Get thermal capacity in J/m3K
    :return: float
    """
    return self._thermal_capacity

  @thermal_capacity.setter
  def thermal_capacity(self, value):
    """
    Set thermal capacity in J/m3K
    :param value: float
    """
    self._thermal_capacity = value

  @property
  def extra_loses_due_to_thermal_bridges(self):
    """
    Get extra loses due to thermal bridges in W/m2K
    :return: float
    """
    return self._extra_loses_due_to_thermal_bridges

  @extra_loses_due_to_thermal_bridges.setter
  def extra_loses_due_to_thermal_bridges(self, value):
    """
    Set extra loses due to thermal bridges in W/m2K
    :param value: float
    """
    self._extra_loses_due_to_thermal_bridges = value

  @property
  def indirect_heated_ratio(self):
    """
    Get indirect heated area ratio
    :return: float
    """
    return self._indirect_heated_ratio

  @indirect_heated_ratio.setter
  def indirect_heated_ratio(self, value):
    """
    Set indirect heated area ratio
    :param value: float
    """
    self._indirect_heated_ratio = value

  @property
  def infiltration_rate_for_ventilation_system_off(self):
    """
    Get infiltration rate for ventilation system off in ACH
    :return: float
    """
    return self._infiltration_rate_for_ventilation_system_off

  @infiltration_rate_for_ventilation_system_off.setter
  def infiltration_rate_for_ventilation_system_off(self, value):
    """
    Set infiltration rate for ventilation system off in ACH
    :param value: float
    """
    self._infiltration_rate_for_ventilation_system_off = value

  @property
  def infiltration_rate_for_ventilation_system_on(self):
    """
    Get infiltration rate for ventilation system on in ACH
    :return: float
    """
    return self._infiltration_rate_for_ventilation_system_on

  @infiltration_rate_for_ventilation_system_on.setter
  def infiltration_rate_for_ventilation_system_on(self, value):
    """
    Set infiltration rate for ventilation system on in ACH
    :param value: float
    """
    self._infiltration_rate_for_ventilation_system_on = value

  @property
  def infiltration_rate_area_for_ventilation_system_off(self):
    """
    Get infiltration rate for ventilation system off in l/s/m2
    :return: float
    """
    return self._infiltration_rate_for_ventilation_system_off

  @infiltration_rate_area_for_ventilation_system_off.setter
  def infiltration_rate_area_for_ventilation_system_off(self, value):
    """
    Set infiltration rate for ventilation system off in l/s/m2
    :param value: float
    """
    self._infiltration_rate_for_ventilation_system_off = value

  @property
  def infiltration_rate_area_for_ventilation_system_on(self):
    """
    Get infiltration rate for ventilation system on in l/s/m2
    :return: float
    """
    return self._infiltration_rate_for_ventilation_system_on

  @infiltration_rate_area_for_ventilation_system_on.setter
  def infiltration_rate_area_for_ventilation_system_on(self, value):
    """
    Set infiltration rate for ventilation system on in l/s/m2
    :param value: float
    """
    self._infiltration_rate_for_ventilation_system_on = value



"""
Electrical storage system
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Saeed Ranjbar saeed.ranjbar@concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.city_model_structure.energy_systems.energy_storage_system import EnergyStorageSystem


class ElectricalStorageSystem(EnergyStorageSystem):
  """"
  Electrical Storage System Class
  """

  def __init__(self):

    super().__init__()
    self._rated_output_power = None
    self._nominal_efficiency = None
    self._battery_voltage = None
    self._depth_of_discharge = None
    self._self_discharge_rate = None

  @property
  def rated_output_power(self):
    """
    Get the rated output power of storage system in Watts
    :return: float
    """
    return self._rated_output_power

  @rated_output_power.setter
  def rated_output_power(self, value):
    """
    Set the rated output power of storage system in Watts
    :param value: float
    """
    self._rated_output_power = value

  @property
  def nominal_efficiency(self):
    """
    Get the nominal efficiency of the storage system
    :return: float
    """
    return self._nominal_efficiency

  @nominal_efficiency.setter
  def nominal_efficiency(self, value):
    """
    Set the nominal efficiency of the storage system
    :param value: float
    """
    self._nominal_efficiency = value

  @property
  def battery_voltage(self):
    """
    Get the battery voltage in Volts
    :return: float
    """
    return self._battery_voltage

  @battery_voltage.setter
  def battery_voltage(self, value):
    """
    Set the battery voltage in Volts
    :param value: float
    """
    self._battery_voltage = value

  @property
  def depth_of_discharge(self):
    """
    Get the depth of discharge as a percentage
    :return: float
    """
    return self._depth_of_discharge

  @depth_of_discharge.setter
  def depth_of_discharge(self, value):
    """
    Set the depth of discharge as a percentage
    :param value: float
    """
    self._depth_of_discharge = value

  @property
  def self_discharge_rate(self):
    """
    Get the self discharge rate of battery as a percentage
    :return: float
    """
    return self._self_discharge_rate

  @self_discharge_rate.setter
  def self_discharge_rate(self, value):
    """
    Set the self discharge rate of battery as a percentage
    :param value: float
    """
    self._self_discharge_rate = value



"""
Energy System catalog non PV generation system
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
Code contributors: Saeed Ranjbar saeed.ranjbar@concordia.ca
"""

from typing import Union
from hub.catalog_factories.data_models.energy_systems.performance_curves import PerformanceCurves
from hub.catalog_factories.data_models.energy_systems.generation_system import GenerationSystem


class NonPvGenerationSystem(GenerationSystem):
  """
  Non PV Generation system class
  """

  def __init__(self, system_id, name, system_type, model_name=None, manufacturer=None, fuel_type=None,
               nominal_heat_output=None, maximum_heat_output=None, minimum_heat_output=None, source_medium=None,
               supply_medium=None, heat_efficiency=None, nominal_cooling_output=None, maximum_cooling_output=None,
               minimum_cooling_output=None, cooling_efficiency=None, electricity_efficiency=None,
               source_temperature=None, source_mass_flow=None, nominal_electricity_output=None,
               maximum_heat_supply_temperature=None, minimum_heat_supply_temperature=None,
               maximum_cooling_supply_temperature=None, minimum_cooling_supply_temperature=None, heat_output_curve=None,
               heat_fuel_consumption_curve=None, heat_efficiency_curve=None, cooling_output_curve=None,
               cooling_fuel_consumption_curve=None, cooling_efficiency_curve=None,
               distribution_systems=None, energy_storage_systems=None, domestic_hot_water=False,
               reversible=None, simultaneous_heat_cold=None):
    super().__init__(system_id=system_id, name=name, model_name=model_name, manufacturer=manufacturer,
                     fuel_type=fuel_type, distribution_systems=distribution_systems,
                     energy_storage_systems=energy_storage_systems)
    self._system_type = system_type
    self._nominal_heat_output = nominal_heat_output
    self._maximum_heat_output = maximum_heat_output
    self._minimum_heat_output = minimum_heat_output
    self._heat_efficiency = heat_efficiency
    self._nominal_cooling_output = nominal_cooling_output
    self._maximum_cooling_output = maximum_cooling_output
    self._minimum_cooling_output = minimum_cooling_output
    self._cooling_efficiency = cooling_efficiency
    self._electricity_efficiency = electricity_efficiency
    self._nominal_electricity_output = nominal_electricity_output
    self._source_medium = source_medium
    self._source_temperature = source_temperature
    self._source_mass_flow = source_mass_flow
    self._supply_medium = supply_medium
    self._maximum_heat_supply_temperature = maximum_heat_supply_temperature
    self._minimum_heat_supply_temperature = minimum_heat_supply_temperature
    self._maximum_cooling_supply_temperature = maximum_cooling_supply_temperature
    self._minimum_cooling_supply_temperature = minimum_cooling_supply_temperature
    self._heat_output_curve = heat_output_curve
    self._heat_fuel_consumption_curve = heat_fuel_consumption_curve
    self._heat_efficiency_curve = heat_efficiency_curve
    self._cooling_output_curve = cooling_output_curve
    self._cooling_fuel_consumption_curve = cooling_fuel_consumption_curve
    self._cooling_efficiency_curve = cooling_efficiency_curve
    self._domestic_hot_water = domestic_hot_water
    self._reversible = reversible
    self._simultaneous_heat_cold = simultaneous_heat_cold

  @property
  def system_type(self):
    """
    Get type
    :return: string
    """
    return self._system_type

  @property
  def nominal_heat_output(self):
    """
    Get nominal heat output of heat generation devices in W
    :return: float
    """
    return self._nominal_heat_output

  @property
  def maximum_heat_output(self):
    """
    Get maximum heat output of heat generation devices in W
    :return: float
    """
    return self._maximum_heat_output

  @property
  def minimum_heat_output(self):
    """
    Get minimum heat output of heat generation devices in W
    :return: float
    """
    return self._minimum_heat_output

  @property
  def source_medium(self):
    """
    Get source_type from [air, water, ground, district_heating, grid, on_site_electricity]
    :return: string
    """
    return self._source_medium

  @property
  def supply_medium(self):
    """
    Get the supply medium from ['air', 'water']
    :return: string
    """
    return self._supply_medium

  @property
  def heat_efficiency(self):
    """
    Get heat_efficiency
    :return: float
    """
    return self._heat_efficiency

  @property
  def nominal_cooling_output(self):
    """
    Get nominal cooling output of heat generation devices in W
    :return: float
    """
    return self._nominal_cooling_output

  @property
  def maximum_cooling_output(self):
    """
    Get maximum heat output of heat generation devices in W
    :return: float
    """
    return self._maximum_cooling_output

  @property
  def minimum_cooling_output(self):
    """
    Get minimum heat output of heat generation devices in W
    :return: float
    """
    return self._minimum_cooling_output

  @property
  def cooling_efficiency(self):
    """
    Get cooling_efficiency
    :return: float
    """
    return self._cooling_efficiency

  @property
  def electricity_efficiency(self):
    """
    Get electricity_efficiency
    :return: float
    """
    return self._electricity_efficiency

  @property
  def source_temperature(self):
    """
    Get source_temperature in degree Celsius
    :return: float
    """
    return self._source_temperature

  @property
  def source_mass_flow(self):
    """
    Get source_mass_flow in kg/s
    :return: float
    """
    return self._source_mass_flow

  @property
  def nominal_electricity_output(self):
    """
    Get nominal_power_output of electricity generation devices or inverters in W
    :return: float
    """
    return self._nominal_electricity_output

  @property
  def maximum_heat_supply_temperature(self):
    """
    Get the maximum heat supply temperature in degree Celsius
    :return: float
    """
    return self._minimum_heat_supply_temperature

  @property
  def minimum_heat_supply_temperature(self):
    """
    Get the minimum heat supply temperature in degree Celsius
    :return: float
    """
    return self._minimum_heat_supply_temperature

  @property
  def maximum_cooling_supply_temperature(self):
    """
    Get the maximum cooling supply temperature in degree Celsius
    :return: float
    """
    return self._maximum_cooling_supply_temperature

  @property
  def minimum_cooling_supply_temperature(self):
    """
    Get the minimum cooling supply temperature in degree Celsius
    :return: float
    """
    return self._minimum_cooling_supply_temperature

  @property
  def heat_output_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heat output curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._heat_output_curve

  @property
  def heat_fuel_consumption_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heating fuel consumption curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._heat_fuel_consumption_curve

  @property
  def heat_efficiency_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heating efficiency curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._heat_efficiency_curve

  @property
  def cooling_output_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heat output curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._cooling_output_curve

  @property
  def cooling_fuel_consumption_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heating fuel consumption curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._cooling_fuel_consumption_curve

  @property
  def cooling_efficiency_curve(self) -> Union[None, PerformanceCurves]:
    """
    Get the heating efficiency curve of the heat generation device
    :return: PerformanceCurve
    """
    return self._cooling_efficiency_curve

  @property
  def domestic_hot_water(self):
    """
    Get the ability to produce domestic hot water
    :return: bool
    """
    return self._domestic_hot_water

  @property
  def reversibility(self):
    """
    Get the ability to produce heating and cooling
    :return: bool
    """
    return self._reversible

  @property
  def simultaneous_heat_cold(self):
    """
    Get the ability to produce heating and cooling at the same time
    :return: bool
    """
    return self._simultaneous_heat_cold

  def to_dictionary(self):
    """Class content to dictionary"""
    _distribution_systems = [_distribution_system.to_dictionary() for _distribution_system in
                             self.distribution_systems] if self.distribution_systems is not None else None
    _energy_storage_systems = [_energy_storage_system.to_dictionary() for _energy_storage_system in
                               self.energy_storage_systems] if self.energy_storage_systems is not None else None
    _heat_output_curve = self.heat_output_curve.to_dictionary() if (
        self.heat_output_curve is not None) else None
    _heat_fuel_consumption_curve = self.heat_fuel_consumption_curve.to_dictionary() if (
        self.heat_fuel_consumption_curve is not None) else None
    _heat_efficiency_curve = self.heat_efficiency_curve.to_dictionary() if (
        self.heat_efficiency_curve is not None) else None
    _cooling_output_curve = self.cooling_output_curve.to_dictionary() if (
        self.cooling_output_curve is not None) else None
    _cooling_fuel_consumption_curve = self.cooling_fuel_consumption_curve.to_dictionary() if (
        self.cooling_fuel_consumption_curve is not None) else None
    _cooling_efficiency_curve = self.cooling_efficiency_curve.to_dictionary() if (
        self.cooling_efficiency_curve is not None) else None

    content = {
      'Energy Generation component':
      {
        'id': self.id,
        'model name': self.model_name,
        'manufacturer': self.manufacturer,
        'type': self.system_type,
        'fuel type': self.fuel_type,
        'nominal heat output [W]': self.nominal_heat_output,
        'maximum heat output [W]': self.maximum_heat_output,
        'minimum heat output [W]': self.minimum_heat_output,
        'source medium': self.source_medium,
        'supply medium': self.supply_medium,
        'source temperature [Celsius]': self.source_temperature,
        'source mass flow [kg/s]': self.source_mass_flow,
        'heat efficiency': self.heat_efficiency,
        'nominal cooling output [W]': self.nominal_cooling_output,
        'maximum cooling output [W]': self.maximum_cooling_output,
        'minimum cooling output [W]': self.minimum_cooling_output,
        'cooling efficiency': self.cooling_efficiency,
        'electricity efficiency': self.electricity_efficiency,
        'nominal power output [W]': self.nominal_electricity_output,
        'maximum heating supply temperature [Celsius]': self.maximum_heat_supply_temperature,
        'minimum heating supply temperature [Celsius]': self.minimum_heat_supply_temperature,
        'maximum cooling supply temperature [Celsius]': self.maximum_cooling_supply_temperature,
        'minimum cooling supply temperature [Celsius]': self.minimum_cooling_supply_temperature,
        'heat output curve': self.heat_output_curve,
        'heat fuel consumption curve': self.heat_fuel_consumption_curve,
        'heat efficiency curve': _heat_efficiency_curve,
        'cooling output curve': self.cooling_output_curve,
        'cooling fuel consumption curve': self.cooling_fuel_consumption_curve,
        'cooling efficiency curve': self.cooling_efficiency_curve,
        'distribution systems connected': _distribution_systems,
        'storage systems connected': _energy_storage_systems,
        'domestic hot water production capability': self.domestic_hot_water,
        'reversible cycle': self.reversibility,
        'simultaneous heat and cooling production': self.simultaneous_heat_cold
      }
    }
    return content



"""
Nrel construction catalog
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez guillermo.gutierrezmorote@concordia.ca
"""

from pathlib import Path
import xmltodict
from hub.catalog_factories.catalog import Catalog
from hub.catalog_factories.data_models.construction.window import Window
from hub.catalog_factories.data_models.construction.material import Material
from hub.catalog_factories.data_models.construction.layer import Layer
from hub.catalog_factories.data_models.construction.construction import Construction
from hub.catalog_factories.data_models.construction.content import Content
from hub.catalog_factories.data_models.construction.archetype import Archetype
from hub.catalog_factories.construction.construction_helper import ConstructionHelper
import hub.helpers.constants as cte


class NrelCatalog(Catalog):
  """
  Nrel catalog class
  """
  def __init__(self, path):
    archetypes_path = str(Path(path / 'us_archetypes.xml').resolve())
    constructions_path = str(Path(path / 'us_constructions.xml').resolve())
    with open(constructions_path, 'r', encoding='utf-8') as xml:
      self._constructions = xmltodict.parse(xml.read(), force_list=('material', 'window', 'construction', 'layer'))
    with open(archetypes_path, 'r', encoding='utf-8') as xml:
      self._archetypes = xmltodict.parse(xml.read(), force_list=('archetype', 'construction'))
    self._catalog_windows = self._load_windows()
    self._catalog_materials = self._load_materials()
    self._catalog_constructions = self._load_constructions()
    self._catalog_archetypes = self._load_archetypes()

    # store the full catalog data model in self._content
    self._content = Content(self._catalog_archetypes,
                            self._catalog_constructions,
                            self._catalog_materials,
                            self._catalog_windows)

  def _load_windows(self):
    _catalog_windows = []
    windows = self._constructions['library']['windows']['window']
    for window in windows:
      frame_ratio = float(window['frame_ratio']['#text'])
      g_value = window['shgc']
      overall_u_value = float(window['conductivity']['#text']) / float(window['thickness']['#text'])
      name = window['@name']
      window_id = window['@id']
      _catalog_windows.append(Window(window_id, frame_ratio, g_value, overall_u_value, name))
    return _catalog_windows

  def _load_materials(self):
    _catalog_materials = []
    materials = self._constructions['library']['materials']['material']
    for material in materials:
      material_id = material['@id']
      name = material['@name']
      solar_absorptance = float(material['solar_absorptance']['#text'])
      thermal_absorptance = float(material['thermal_absorptance']['#text'])
      visible_absorptance = float(material['visible_absorptance']['#text'])
      no_mass = False
      thermal_resistance = None
      conductivity = None
      density = None
      specific_heat = None
      if 'no_mass' in material and material['no_mass'] == 'true':
        no_mass = True
        thermal_resistance = float(material['thermal_resistance']['#text'])
      else:
        conductivity = float(material['conductivity']['#text'])
        density = float(material['density']['#text'])
        specific_heat = float(material['specific_heat']['#text'])
      _material = Material(material_id,
                           name,
                           solar_absorptance,
                           thermal_absorptance,
                           visible_absorptance,
                           no_mass,
                           thermal_resistance,
                           conductivity,
                           density,
                           specific_heat)
      _catalog_materials.append(_material)
    return _catalog_materials

  def _load_constructions(self):
    _catalog_constructions = []
    constructions = self._constructions['library']['constructions']['construction']
    for construction in constructions:
      construction_id = construction['@id']
      construction_type = ConstructionHelper().nrel_surfaces_types_to_hub_types[construction['@type']]
      name = construction['@name']
      layers = []
      for layer in construction['layers']['layer']:
        layer_id = layer['@id']
        layer_name = layer['@name']
        material_id = layer['material'][0]
        thickness = 0
        if 'thickness' in layer:
          thickness = float(layer['thickness']['#text'])
        for material in self._catalog_materials:
          if str(material_id) == str(material.id):
            layers.append(Layer(layer_id, layer_name, material, thickness))
            break
      _catalog_constructions.append(Construction(construction_id, construction_type, name, layers))
    return _catalog_constructions

  def _load_archetypes(self):
    _catalog_archetypes = []
    archetypes = self._archetypes['archetypes']['archetype']
    for archetype in archetypes:
      archetype_id = archetype['@id']
      function = archetype['@building_type']
      name = f"{function} {archetype['@climate_zone']} {archetype['@reference_standard']}"
      climate_zone = archetype['@climate_zone']
      construction_period = ConstructionHelper().reference_standard_to_construction_period[
        archetype['@reference_standard']
      ]
      average_storey_height = float(archetype['average_storey_height']['#text'])
      thermal_capacity = float(archetype['thermal_capacity']['#text']) * 1000
      extra_loses_due_to_thermal_bridges = float(archetype['extra_loses_due_to_thermal_bridges']['#text'])
      indirect_heated_ratio = float(archetype['indirect_heated_ratio']['#text'])
      infiltration_rate_for_ventilation_system_off = float(
        archetype['infiltration_rate_for_ventilation_system_off']['#text']
      ) / cte.HOUR_TO_SECONDS
      infiltration_rate_for_ventilation_system_on = float(
        archetype['infiltration_rate_for_ventilation_system_on']['#text']
      ) / cte.HOUR_TO_SECONDS

      archetype_constructions = []
      for archetype_construction in archetype['constructions']['construction']:
        for construction in self._catalog_constructions:
          if construction.id == archetype_construction['@id']:
            window_ratio = float(archetype_construction['window_ratio']['#text'])
            window_id = archetype_construction['window']
            _construction = None
            _window = None
            for window in self._catalog_windows:
              if window_id == window.id:
                _window = window
                break
            _construction = Construction(construction.id,
                                         construction.type,
                                         construction.name,
                                         construction.layers,
                                         window_ratio,
                                         _window)
            archetype_constructions.append(_construction)
            break

      _catalog_archetypes.append(Archetype(archetype_id,
                                           name,
                                           function,
                                           climate_zone,
                                           construction_period,
                                           archetype_constructions,
                                           average_storey_height,
                                           thermal_capacity,
                                           extra_loses_due_to_thermal_bridges,
                                           indirect_heated_ratio,
                                           infiltration_rate_for_ventilation_system_off,
                                           infiltration_rate_for_ventilation_system_on,
                                           0,
                                           0))
    return _catalog_archetypes

  def names(self, category=None):
    """
    Get the catalog elements names
    :parm: optional category filter
    """
    if category is None:
      _names = {'archetypes': [], 'constructions': [], 'materials': [], 'windows': []}
      for archetype in self._content.archetypes:
        _names['archetypes'].append(archetype.name)
      for construction in self._content.constructions:
        _names['constructions'].append(construction.name)
      for material in self._content.materials:
        _names['materials'].append(material.name)
      for window in self._content.windows:
        _names['windows'].append(window.name)
    else:
      _names = {category: []}
      if category.lower() == 'archetypes':
        for archetype in self._content.archetypes:
          _names[category].append(archetype.name)
      elif category.lower() == 'constructions':
        for construction in self._content.constructions:
          _names[category].append(construction.name)
      elif category.lower() == 'materials':
        for material in self._content.materials:
          _names[category].append(material.name)
      elif category.lower() == 'windows':
        for window in self._content.windows:
          _names[category].append(window.name)
      else:
        raise ValueError(f'Unknown category [{category}]')
    return _names

  def entries(self, category=None):
    """
    Get the catalog elements
    :parm: optional category filter
    """
    if category is None:
      return self._content
    if category.lower() == 'archetypes':
      return self._content.archetypes
    if category.lower() == 'constructions':
      return self._content.constructions
    if category.lower() == 'materials':
      return self._content.materials
    if category.lower() == 'windows':
      return self._content.windows
    raise ValueError(f'Unknown category [{category}]')

  def get_entry(self, name):
    """
    Get one catalog element by names
    :parm: entry name
    """
    for entry in self._content.archetypes:
      if entry.name.lower() == name.lower():
        return entry
    for entry in self._content.constructions:
      if entry.name.lower() == name.lower():
        return entry
    for entry in self._content.materials:
      if entry.name.lower() == name.lower():
        return entry
    for entry in self._content.windows:
      if entry.name.lower() == name.lower():
        return entry
    raise IndexError(f"{name} doesn't exists in the catalog")



"""
Test db factory
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Peter Yefi peteryefi@gmail.com
"""
import distutils
import glob
import os
import subprocess
import unittest
from pathlib import Path
from unittest import TestCase

import sqlalchemy
import sqlalchemy.exc
from sqlalchemy import create_engine
from sqlalchemy_utils import database_exists, create_database, drop_database

import hub.helpers.constants as cte
from hub.exports.energy_building_exports_factory import EnergyBuildingsExportsFactory
from hub.exports.exports_factory import ExportsFactory
from hub.helpers.data.montreal_function_to_hub_function import MontrealFunctionToHubFunction
from hub.imports.construction_factory import ConstructionFactory
from hub.imports.energy_systems_factory import EnergySystemsFactory
from hub.imports.geometry_factory import GeometryFactory
from hub.imports.results_factory import ResultFactory
from hub.imports.usage_factory import UsageFactory
from hub.imports.weather_factory import WeatherFactory
from hub.persistence.db_control import DBControl
from hub.persistence.models import City, Application, CityObject, SimulationResults, UserRoles
from hub.persistence.models import User
from hub.persistence.repository import Repository


class Control:
  _skip_test = False
  _skip_reason = 'PostgreSQL not properly installed in host machine'

  def __init__(self):
    """
    Test
    setup
    :return: None
    """
    self._skip_test = False
    # Create test database
    dotenv_path = Path("{}/.local/etc/hub/.env".format(os.path.expanduser('~'))).resolve()
    if not dotenv_path.exists():
      self._skip_test = True
      self._skip_reason = f'.env file missing at {dotenv_path}'
      return
    dotenv_path = str(dotenv_path)
    repository = Repository(db_name='persistence_test_db', app_env='TEST', dotenv_path=dotenv_path)
    engine = create_engine(repository.configuration.connection_string)
    if database_exists(engine.url):
      drop_database(engine.url)
    create_database(engine.url)
    Application.__table__.create(bind=engine, checkfirst=True)
    User.__table__.create(bind=engine, checkfirst=True)
    City.__table__.create(bind=engine, checkfirst=True)
    CityObject.__table__.create(bind=engine, checkfirst=True)
    SimulationResults.__table__.create(bind=engine, checkfirst=True)

    self._database = DBControl(
      db_name=repository.configuration.db_name,
      app_env='TEST',
      dotenv_path=dotenv_path)

    example_path = (Path(__file__).parent / 'tests_data').resolve()
    city_file = (example_path / 'test.geojson').resolve()
    output_path = (Path(__file__).parent / 'tests_outputs').resolve()
    self._application_uuid = '60b7fc1b-f389-4254-9ffd-22a4cf32c7a3'
    self._application_id = 1
    self._user_id = 1
    self._pickle_path = 'tests_data/pickle_path.bz2'
    self._city = GeometryFactory('geojson',
                                 city_file,
                                 height_field='citygml_me',
                                 year_of_construction_field='ANNEE_CONS',
                                 aliases_field=['ID_UEV', 'CIVIQUE_DE', 'NOM_RUE'],
                                 function_field='CODE_UTILI',
                                 function_to_hub=MontrealFunctionToHubFunction().dictionary).city

    ConstructionFactory('nrcan', self._city).enrich()
    UsageFactory('nrcan', self._city).enrich()
    WeatherFactory('epw', self._city).enrich()
    ExportsFactory('sra', self._city, output_path).export()
    sra_file = str((output_path / f'{self._city.name}_sra.xml').resolve())
    subprocess.run([self.sra, sra_file], stdout=subprocess.DEVNULL)
    ResultFactory('sra', self._city, output_path).enrich()

    for building in self._city.buildings:
      building.energy_systems_archetype_name = 'system 1 gas pv'
    EnergySystemsFactory('montreal_custom', self._city).enrich()
    EnergyBuildingsExportsFactory('insel_monthly_energy_balance', self._city, output_path).export()
    _insel_files = glob.glob(f'{output_path}/*.insel')
    for insel_file in _insel_files:
      subprocess.run([self.insel, str(insel_file)], stdout=subprocess.DEVNULL)
    ResultFactory('insel_monthly_energy_balance', self._city, output_path).enrich()

    self._database = DBControl(
      db_name=repository.configuration.db_name,
      app_env='TEST',
      dotenv_path=dotenv_path)

    self._application_uuid = 'b9e0ce80-1218-410c-8a64-9d9b7026aad8'
    self._application_id = 1
    self._user_id = 1
    try:
      self._application_id = self._database.persist_application(
        'test',
        'test',
        self.application_uuid
      )
    except sqlalchemy.exc.SQLAlchemyError:
      self._application_id = self._database.application_info(self.application_uuid).id

    try:
      self._user_id = self._database.create_user('test', self._application_id, 'test', UserRoles.Admin)
    except sqlalchemy.exc.SQLAlchemyError:
      self._user_id = self._database.user_info(name='test', password='test', application_id=self._application_id).id

    self._pickle_path = (example_path / 'pickle_path.bz2').resolve()

    self._city

  @property
  def database(self):
    return self._database

  @property
  def application_uuid(self):
    return self._application_uuid

  @property
  def application_id(self):
    return self._application_id

  @property
  def user_id(self):
    return self._user_id

  @property
  def skip_test(self):
    return self._skip_test

  @property
  def insel(self):
    return distutils.spawn.find_executable('insel')

  @property
  def sra(self):
    return distutils.spawn.find_executable('sra')

  @property
  def skip_insel_test(self):
    return self.insel is None

  @property
  def skip_reason(self):
    return self._skip_reason

  @property
  def message(self):
    return self._skip_reason

  @property
  def pickle_path(self):
    return self._pickle_path

  @property
  def city(self):
    return self._city


control = Control()


class TestDBFactory(TestCase):
  """
TestDBFactory
"""

  @unittest.skipIf(control.skip_test, control.skip_reason)
  def test_save_city(self):
    control.city.name = "Montreal"
    city_id = control.database.persist_city(
      control.city,
      control.pickle_path,
      control.city.name,
      control.application_id,
      control.user_id)
    control.database.delete_city(city_id)

  @unittest.skipIf(control.skip_test, control.skip_reason)
  def test_get_update_city(self):
    city_id = control.database.persist_city(control.city,
                                            control.pickle_path,
                                            control.city.name,
                                            control.application_id,
                                            control.user_id)
    control.city.name = "Ottawa"
    control.database.update_city(city_id, control.city)
    cities = control.database.cities_by_user_and_application(
      control.user_id,
      control.application_id)
    for updated_city in cities:
      if updated_city.id == city_id:
        self.assertEqual(updated_city.name, control.city.name)
        break
    control.database.delete_city(city_id)

  @unittest.skipIf(control.skip_test, control.skip_reason)
  @unittest.skipIf(control.skip_insel_test, 'insel is not installed')
  def test_save_results(self):
    city_id = control.database.persist_city(control.city,
                                            control.pickle_path,
                                            'current status',
                                            control.application_id,
                                            control.user_id)
    city_objects_id = []
    request_values = {
      'scenarios': [{'current status': ['1']}]
    }
    for building in control.city.buildings:
      _building = control.database.building_info(building.name, city_id)
      if cte.MONTH not in building.cooling_demand:
        print(f'building {building.name} not calculated')
        continue
      monthly_cooling_peak_load = building.cooling_peak_load[cte.MONTH]
      yearly_cooling_peak_load = building.cooling_peak_load[cte.YEAR]
      monthly_heating_peak_load = building.heating_peak_load[cte.MONTH]
      yearly_heating_peak_load = building.heating_peak_load[cte.YEAR]
      monthly_lighting_peak_load = building.lighting_peak_load[cte.MONTH]
      yearly_lighting_peak_load = building.lighting_peak_load[cte.YEAR]
      monthly_appliances_peak_load = building.appliances_peak_load[cte.MONTH]
      yearly_appliances_peak_load = building.appliances_peak_load[cte.YEAR]
      monthly_cooling_demand = building.cooling_demand[cte.MONTH]
      yearly_cooling_demand = building.cooling_demand[cte.YEAR]
      monthly_heating_demand = building.heating_demand[cte.MONTH]
      yearly_heating_demand = building.heating_demand[cte.YEAR]
      monthly_lighting_electrical_demand = building.lighting_electrical_demand[cte.MONTH]
      yearly_lighting_electrical_demand = building.lighting_electrical_demand[cte.YEAR]
      monthly_appliances_electrical_demand = building.appliances_electrical_demand[cte.MONTH]
      yearly_appliances_electrical_demand = building.appliances_electrical_demand[cte.YEAR]
      monthly_domestic_hot_water_heat_demand = building.domestic_hot_water_heat_demand[cte.MONTH]
      yearly_domestic_hot_water_heat_demand = building.domestic_hot_water_heat_demand[cte.YEAR]
      monthly_heating_consumption = building.heating_consumption[cte.MONTH]
      yearly_heating_consumption = building.heating_consumption[cte.YEAR]
      monthly_cooling_consumption = building.cooling_consumption[cte.MONTH]
      yearly_cooling_consumption = building.cooling_consumption[cte.YEAR]
      monthly_domestic_hot_water_consumption = building.domestic_hot_water_consumption[cte.MONTH]
      yearly_domestic_hot_water_consumption = building._domestic_hot_water_consumption[cte.YEAR]
      monthly_distribution_systems_electrical_consumption = building.distribution_systems_electrical_consumption[
        cte.MONTH]
      yearly_distribution_systems_electrical_consumption = building.distribution_systems_electrical_consumption[
        cte.YEAR]
      monthly_on_site_electrical_production = [x * cte.WATTS_HOUR_TO_JULES
                                               for x in building.onsite_electrical_production[cte.MONTH]]
      yearly_on_site_electrical_production = [x * cte.WATTS_HOUR_TO_JULES
                                              for x in building.onsite_electrical_production[cte.YEAR]]
      results = {cte.INSEL_MEB: {
        'monthly_cooling_peak_load': monthly_cooling_peak_load,
        'yearly_cooling_peak_load': yearly_cooling_peak_load,
        'monthly_heating_peak_load': monthly_heating_peak_load,
        'yearly_heating_peak_load': yearly_heating_peak_load,
        'monthly_lighting_peak_load': monthly_lighting_peak_load,
        'yearly_lighting_peak_load': yearly_lighting_peak_load,
        'monthly_appliances_peak_load': monthly_appliances_peak_load,
        'yearly_appliances_peak_load': yearly_appliances_peak_load,
        'monthly_cooling_demand': monthly_cooling_demand,
        'yearly_cooling_demand': yearly_cooling_demand,
        'monthly_heating_demand': monthly_heating_demand,
        'yearly_heating_demand': yearly_heating_demand,
        'monthly_lighting_electrical_demand': monthly_lighting_electrical_demand,
        'yearly_lighting_electrical_demand': yearly_lighting_electrical_demand,
        'monthly_appliances_electrical_demand': monthly_appliances_electrical_demand,
        'yearly_appliances_electrical_demand': yearly_appliances_electrical_demand,
        'monthly_domestic_hot_water_heat_demand': monthly_domestic_hot_water_heat_demand,
        'yearly_domestic_hot_water_heat_demand': yearly_domestic_hot_water_heat_demand,
        'monthly_heating_consumption': monthly_heating_consumption,
        'yearly_heating_consumption': yearly_heating_consumption,
        'monthly_cooling_consumption': monthly_cooling_consumption,
        'yearly_cooling_consumption': yearly_cooling_consumption,
        'monthly_domestic_hot_water_consumption': monthly_domestic_hot_water_consumption,
        'yearly_domestic_hot_water_consumption': yearly_domestic_hot_water_consumption,
        'monthly_distribution_systems_electrical_consumption': monthly_distribution_systems_electrical_consumption,
        'yearly_distribution_systems_electrical_consumption': yearly_distribution_systems_electrical_consumption,
        'monthly_on_site_electrical_production': monthly_on_site_electrical_production,
        'yearly_on_site_electrical_production': yearly_on_site_electrical_production
      }}

      db_building_id = _building.id
      city_objects_id.append(db_building_id)
      control.database.add_simulation_results(
        cte.INSEL_MEB,
        results, city_object_id=db_building_id)
    self.assertEqual(17, len(city_objects_id), 'wrong number of results')
    self.assertIsNotNone(city_objects_id[0], 'city_object_id is None')
    results = control.database.results(control.user_id, control.application_id, request_values)
    self.assertEqual(
      28,
      len(results['current status'][0]['insel meb'].keys()),
      'wrong number of results after retrieve'
    )

    for _id in city_objects_id:
      control.database.delete_results_by_name('insel meb', city_object_id=_id)
    control.database.delete_city(city_id)

  @classmethod
  @unittest.skipIf(control.skip_test, control.skip_reason)
  def tearDownClass(cls):
    control.database.delete_application(control.application_uuid)
    control.database.delete_user(control.user_id)
    os.unlink(control.pickle_path)
    output_files = glob.glob('./tests_outputs/*')
    for output_file in output_files:
      os.unlink(output_file)



"""
Simulation results repository with database CRUD operations
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""
import datetime
import logging

from sqlalchemy import or_
from sqlalchemy import select
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.orm import Session

from hub.persistence.repository import Repository
from hub.persistence.models import City
from hub.persistence.models import CityObject
from hub.persistence.models import SimulationResults as Model


class SimulationResults(Repository):
  """
  Simulation results repository
  """
  _instance = None

  def __init__(self, db_name: str, dotenv_path: str, app_env: str):
    super().__init__(db_name, dotenv_path, app_env)

  def __new__(cls, db_name, dotenv_path, app_env):
    """
    Implemented for a singleton pattern
    """
    if cls._instance is None:
      cls._instance = super(SimulationResults, cls).__new__(cls)
    return cls._instance

  def insert(self, name: str, values: str, city_id=None, city_object_id=None):
    """
    Inserts simulations results linked either with a city as a whole or with a city object
    :param name: results name
    :param values: the simulation results in json format
    :param city_id: optional city id
    :param city_object_id: optional city object id
    :return: Identity id
    """
    if city_id is not None:
      _ = self._get_city(city_id)
    else:
      _ = self._get_city_object(city_object_id)
    try:
      simulation_result = Model(name=name,
                                values=values,
                                city_id=city_id,
                                city_object_id=city_object_id)
      with Session(self.engine) as session:
        session.add(simulation_result)
        session.flush()
        session.commit()
        session.refresh(simulation_result)
        return simulation_result.id
    except SQLAlchemyError as err:
      logging.error('An error occurred while creating city_object %s', err)
      raise SQLAlchemyError from err

  def update(self, name: str, values: str, city_id=None, city_object_id=None):
    """
    Updates simulation results for a city or a city object
    :param name: The simulation results tool and workflow name
    :param values: the simulation results in json format
    :param city_id: optional city id
    :param city_object_id: optional city object id
    :return: None
    """
    try:
      with Session(self.engine) as session:
        if city_id is not None:
          session.query(Model).filter(Model.name == name, Model.city_id == city_id).update(
          {
            'values': values,
            'updated': datetime.datetime.utcnow()
          })
          session.commit()
        elif city_object_id is not None:
          session.query(Model).filter(Model.name == name, Model.city_object_id == city_object_id).update(
            {
              'values': values,
              'updated': datetime.datetime.utcnow()
            })
          session.commit()
        else:
          raise NotImplementedError('Missing either city_id or city_object_id')
    except SQLAlchemyError as err:
      logging.error('Error while updating city object %s', err)
      raise SQLAlchemyError from err

  def delete(self, name: str, city_id=None, city_object_id=None):
    """
    Deletes an application with the application_uuid
    :param name: The simulation results tool and workflow name
    :param city_id: The id for the city owning the simulation results
    :param city_object_id: the id for the city_object owning these simulation results
    :return: None
    """
    try:
      with Session(self.engine) as session:
        if city_id is not None:
          session.query(Model).filter(Model.name == name, Model.city_id == city_id).delete()
          session.commit()
        elif city_object_id is not None:
          session.query(Model).filter(Model.name == name, Model.city_object_id == city_object_id).delete()
          session.commit()
        else:
          raise NotImplementedError('Missing either city_id or city_object_id')
    except SQLAlchemyError as err:
      logging.error('Error while deleting application: %s', err)
      raise SQLAlchemyError from err

  def _get_city(self, city_id) -> City:
    """
    Fetch a city object based city id
    :param city_id: a city identifier
    :return: [City] with the provided city_id
    """
    try:
      with Session(self.engine) as session:
        return session.execute(select(City).where(City.id == city_id)).first()
    except SQLAlchemyError as err:
      logging.error('Error while fetching city by city_id: %s', err)
      raise SQLAlchemyError from err

  def _get_city_object(self, city_object_id) -> [CityObject]:
    """
    Fetch a city object based city id
    :param city_object_id: a city object identifier
    :return: [CityObject] with the provided city_object_id
    """
    try:
      with Session(self.engine) as session:
        return session.execute(select(CityObject).where(CityObject.id == city_object_id)).first()
    except SQLAlchemyError as err:
      logging.error('Error while fetching city by city_id: %s', err)
      raise SQLAlchemyError from err

  def get_simulation_results_by_city_id_city_object_id_and_names(self, city_id, city_object_id, result_names=None) -> [Model]:
    """
    Fetch the simulation results based in the city_id or city_object_id with the given names or all
    :param city_id: the city id
    :param city_object_id: the city object id
    :param result_names: if given filter the results
    :return: [SimulationResult]
    """
    try:
      with Session(self.engine) as session:
        result_set = session.execute(select(Model).where(or_(
          Model.city_id == city_id,
          Model.city_object_id == city_object_id
        )))
        results = [r[0] for r in result_set]
        if not result_names:
          return results
        filtered_results = []
        for result in results:
          if result.name in result_names:
            filtered_results.append(result)
        return filtered_results
    except SQLAlchemyError as err:
      logging.error('Error while fetching city by city_id: %s', err)
      raise SQLAlchemyError from err

  def get_simulation_results_by_city_object_id_and_names(self, city_object_id, result_names=None) -> [Model]:
    """
    Fetch the simulation results based in the city_object_id with the given names or all
    :param city_object_id: the city object id
    :param result_names: if given filter the results
    :return: [SimulationResult]
    """
    try:
      with Session(self.engine) as session:
        result_set = session.execute(select(Model).where(
          Model.city_object_id == city_object_id
        ))
        results = [r[0] for r in result_set]
        if not result_names:
          return results
        filtered_results = []
        for result in results:
          if result.name in result_names:
            filtered_results.append(result)
        return filtered_results
    except SQLAlchemyError as err:
      logging.error('Error while fetching city by city_id: %s', err)
      raise SQLAlchemyError from err


"""
Model representation of a city object
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guille Gutierrez Guillermo.GutierrezMorote@concordia.ca
"""

import datetime
import logging

from sqlalchemy import Column, Integer, String, Sequence, ForeignKey, Float
from sqlalchemy import DateTime

from hub.city_model_structure.building import Building
from hub.persistence.configuration import Models


class CityObject(Models):
  """
  A model representation of an application
  """
  __tablename__ = 'city_object'
  id = Column(Integer, Sequence('city_object_id_seq'), primary_key=True)
  city_id = Column(Integer, ForeignKey('city.id', ondelete='CASCADE'), nullable=False)
  name = Column(String, nullable=False)
  aliases = Column(String, nullable=True)
  type = Column(String, nullable=False)
  year_of_construction = Column(Integer, nullable=True)
  function = Column(String, nullable=True)
  usage = Column(String, nullable=True)
  volume = Column(Float, nullable=False)
  area = Column(Float, nullable=False)
  total_heating_area = Column(Float, nullable=False)
  wall_area = Column(Float, nullable=False)
  windows_area = Column(Float, nullable=False)
  roof_area = Column(Float, nullable=False)
  total_pv_area = Column(Float, nullable=False)
  system_name = Column(String, nullable=False)
  created = Column(DateTime, default=datetime.datetime.utcnow)
  updated = Column(DateTime, default=datetime.datetime.utcnow)

  def __init__(self, city_id, building: Building):
    self.city_id = city_id
    self.name = building.name
    self.aliases = building.aliases
    self.type = building.type
    self.year_of_construction = building.year_of_construction
    self.function = building.function
    self.usage = building.usages_percentage
    self.volume = building.volume
    self.area = building.floor_area
    self.roof_area = sum(roof.solid_polygon.area for roof in building.roofs)
    self.total_pv_area = sum(roof.solid_polygon.area * roof.solar_collectors_area_reduction_factor for roof in building.roofs)
    storeys = building.storeys_above_ground
    wall_area = 0
    window_ratio = 0
    try:
      if storeys is None:
        storeys = building.max_height / building.average_storey_height
      for internal_zone in building.internal_zones:
        for thermal_zone in internal_zone.thermal_zones_from_internal_zones:
          for thermal_boundary in thermal_zone.thermal_boundaries:
            window_ratio = thermal_boundary.window_ratio
            break
    except TypeError:
      storeys = 0
      logging.warning(
        'building %s has no storey height so heating area, storeys and window ratio cannot be calculated',
        self.name
      )
    self.total_heating_area = building.floor_area * storeys

    for wall in building.walls:
      wall_area += wall.solid_polygon.area
    self.wall_area = wall_area
    self.windows_area = wall_area * window_ratio
    system_name = building.energy_systems_archetype_name
    if system_name is None:
      system_name = ''
    self.system_name = system_name



"""
Energy System catalog heat generation system
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2023 Concordia CERC group
Project Coder Saeed Ranjbar saeed.ranjbar@concordia.ca
Code contributors: Pilar Monsalvete Alvarez de Uribarri pilar.monsalvete@concordia.ca
"""

from hub.catalog_factories.data_models.energy_systems.generation_system import GenerationSystem


class PvGenerationSystem(GenerationSystem):
  """
  Electricity Generation system class
  """

  def __init__(self, system_id, name, system_type, model_name=None, manufacturer=None, electricity_efficiency=None,
               nominal_electricity_output=None, nominal_ambient_temperature=None, nominal_cell_temperature=None,
               nominal_radiation=None, standard_test_condition_cell_temperature=None,
               standard_test_condition_maximum_power=None, standard_test_condition_radiation=None,
               cell_temperature_coefficient=None, width=None, height=None, distribution_systems=None,
               energy_storage_systems=None):
    super().__init__(system_id=system_id, name=name, model_name=model_name,
                     manufacturer=manufacturer, fuel_type='renewable', distribution_systems=distribution_systems,
                     energy_storage_systems=energy_storage_systems)
    self._system_type = system_type
    self._electricity_efficiency = electricity_efficiency
    self._nominal_electricity_output = nominal_electricity_output
    self._nominal_ambient_temperature = nominal_ambient_temperature
    self._nominal_cell_temperature = nominal_cell_temperature
    self._nominal_radiation = nominal_radiation
    self._standard_test_condition_cell_temperature = standard_test_condition_cell_temperature
    self._standard_test_condition_maximum_power = standard_test_condition_maximum_power
    self._standard_test_condition_radiation = standard_test_condition_radiation
    self._cell_temperature_coefficient = cell_temperature_coefficient
    self._width = width
    self._height = height

  @property
  def system_type(self):
    """
    Get type
    :return: string
    """
    return self._system_type

  @property
  def nominal_electricity_output(self):
    """
    Get nominal_power_output of electricity generation devices or inverters in W
    :return: float
    """
    return self._nominal_electricity_output

  @property
  def electricity_efficiency(self):
    """
    Get electricity_efficiency
    :return: float
    """
    return self._electricity_efficiency

  @property
  def nominal_ambient_temperature(self):
    """
    Get nominal ambient temperature of PV panels in degree Celsius
    :return: float
    """
    return self._nominal_ambient_temperature

  @property
  def nominal_cell_temperature(self):
    """
    Get nominal cell temperature of PV panels in degree Celsius
    :return: float
    """
    return self._nominal_cell_temperature

  @property
  def nominal_radiation(self):
    """
    Get nominal radiation of PV panels
    :return: float
    """
    return self._nominal_radiation

  @property
  def standard_test_condition_cell_temperature(self):
    """
    Get standard test condition cell temperature of PV panels in degree Celsius
    :return: float
    """
    return self._standard_test_condition_cell_temperature

  @property
  def standard_test_condition_maximum_power(self):
    """
    Get standard test condition maximum power of PV panels in W
    :return: float
    """
    return self._standard_test_condition_maximum_power

  @property
  def standard_test_condition_radiation(self):
    """
    Get standard test condition cell temperature of PV panels in W/m2
    :return: float
    """
    return self._standard_test_condition_radiation


  @property
  def cell_temperature_coefficient(self):
    """
    Get cell temperature coefficient of PV module
    :return: float
    """
    return self._cell_temperature_coefficient

  @property
  def width(self):
    """
    Get PV module width in m
    :return: float
    """
    return self._width

  @property
  def height(self):
    """
    Get PV module height in m
    :return: float
    """
    return self._height

  def to_dictionary(self):
    """Class content to dictionary"""
    _distribution_systems = [_distribution_system.to_dictionary() for _distribution_system in
                             self.distribution_systems] if self.distribution_systems is not None else None
    _energy_storage_systems = [_energy_storage_system.to_dictionary() for _energy_storage_system in
                               self.energy_storage_systems] if self.energy_storage_systems is not None else None
    content = {
      'Energy Generation component':
      {
        'id': self.id,
        'model name': self.model_name,
        'manufacturer': self.manufacturer,
        'type': self.system_type,
        'fuel type': self.fuel_type,
        'electricity efficiency': self.electricity_efficiency,
        'nominal power output [W]': self.nominal_electricity_output,
        'nominal ambient temperature [Celsius]': self.nominal_ambient_temperature,
        'nominal cell temperature [Celsius]': self.nominal_cell_temperature,
        'nominal radiation [W/m2]': self.nominal_radiation,
        'standard test condition cell temperature [Celsius]': self.standard_test_condition_cell_temperature,
        'standard test condition maximum power [W]': self.standard_test_condition_maximum_power,
        'standard test condition radiation [W/m2]': self.standard_test_condition_radiation,
        'cell temperature coefficient': self.cell_temperature_coefficient,
        'width': self.width,
        'height': self.height,
        'distribution systems connected': _distribution_systems,
        'storage systems connected': _energy_storage_systems
      }
    }
    return content



"""
Dictionaries module for hub usage to Palma usage
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Cecilia Pérez cperez@irec.cat
"""

"""
Codification of uses from cadastre:
U: store-parking. Residential Use
S: store-parking. Industrial Use
V: Residential
I: Industrial
O: Offices
C: Comercial
K: Sportive center
T: Shows
G: Leisure and Hostelry
Y: Health and charity
E: Culture
R: Religion
M: Urbanization work, gardening and undeveloped land
P: Singular building 
B: Farm warehouse
J: Farm Industry
Z: Farm-related
"""

import hub.helpers.constants as cte

class HubUsageToPalmaUsage:
  """
  Hub usage to Palma usage class
  """

  def __init__(self):
    self._dictionary = {
      cte.RESIDENTIAL: 'residential',
      cte.SINGLE_FAMILY_HOUSE: 'residential',
      cte.HIGH_RISE_APARTMENT: 'residential',
      cte.MID_RISE_APARTMENT: 'residential',
      cte.MULTI_FAMILY_HOUSE: 'residential'
    }

  @property
  def dictionary(self) -> dict:
    """
    Get the dictionary
    :return: {}
    """
    return self._dictionary



"""
Simplified Radiosity Algorithm
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Guillermo.GutierrezMorote@concordia.ca
"""

import pandas as pd
import hub.helpers.constants as cte
from hub.helpers.monthly_values import MonthlyValues


class SimplifiedRadiosityAlgorithm:
  """
  Import SRA results
  """
  def __init__(self, city, base_path):

    self._city = city
    self._base_path = base_path
    self._input_file_path = (self._base_path / f'{self._city.name}_sra_SW.out').resolve()
    try:
      self._results = pd.read_csv(self._input_file_path, sep='\s+', header=0).to_dict(orient='list')
    except FileNotFoundError as err:
      raise FileNotFoundError('No SRA output file found') from err

  def enrich(self):
    """
    saves in building surfaces the correspondent irradiance at different time-scales depending on the mode
    if building is None, it saves all buildings' surfaces in file, if building is specified, it saves only that
    specific building values
    :return: none
    """
    for key in self._results:
      _irradiance = {}
      header_name = key.split(':')
      result = [x for x in self._results[key]]
      city_object_name = header_name[1]
      building = self._city.city_object(city_object_name)
      surface_id = header_name[2]
      surface = building.surface_by_id(surface_id)
      monthly_result = MonthlyValues.get_total_month(result)
      yearly_result = [sum(result)]
      _irradiance[cte.YEAR] = yearly_result
      _irradiance[cte.MONTH] = monthly_result
      _irradiance[cte.HOUR] = result
      surface.global_irradiance = _irradiance

    self._city.level_of_detail.surface_radiation = 2
    for building in self._city.buildings:
      building.level_of_detail.surface_radiation = 2



"""
Montreal future energy system catalog
SPDX - License - Identifier: LGPL - 3.0 - or -later
Copyright © 2022 Concordia CERC group
Project Coder Saeed Ranjbar saeed.ranjbar@concordia.ca
"""

import xmltodict
from pathlib import Path
from hub.catalog_factories.catalog import Catalog
from hub.catalog_factories.data_models.energy_systems.distribution_system import DistributionSystem
from hub.catalog_factories.data_models.energy_systems.emission_system import EmissionSystem
from hub.catalog_factories.data_models.energy_systems.system import System
from hub.catalog_factories.data_models.energy_systems.content import Content
from hub.catalog_factories.data_models.energy_systems.non_pv_generation_system import NonPvGenerationSystem
from hub.catalog_factories.data_models.energy_systems.pv_generation_system import PvGenerationSystem
from hub.catalog_factories.data_models.energy_systems.thermal_storage_system import ThermalStorageSystem
from hub.catalog_factories.data_models.energy_systems.performance_curves import PerformanceCurves
from hub.catalog_factories.data_models.energy_systems.archetype import Archetype
from hub.catalog_factories.data_models.construction.material import Material
from hub.catalog_factories.data_models.construction.layer import Layer


class MontrealFutureSystemCatalogue(Catalog):
  """
  North america energy system catalog class
  """

  def __init__(self, path):
    path = str(path / 'montreal_future_systems.xml')
    with open(path, 'r', encoding='utf-8') as xml:
      self._archetypes = xmltodict.parse(xml.read(),
                                         force_list=['pv_generation_component', 'templateStorages', 'demand',
                                                     'system', 'system_id'])

    self._storage_components = self._load_storage_components()
    self._generation_components = self._load_generation_components()
    self._energy_emission_components = self._load_emission_equipments()
    self._distribution_components = self._load_distribution_equipments()
    self._systems = self._load_systems()
    self._system_archetypes = self._load_archetypes()
    self._content = Content(self._system_archetypes,
                            self._systems,
                            generations=self._generation_components,
                            distributions=self._distribution_components)

  def _load_generation_components(self):
    generation_components = []
    non_pv_generation_components = self._archetypes['EnergySystemCatalog']['energy_generation_components'][
      'non_pv_generation_component']
    if non_pv_generation_components is not None:
      for non_pv in non_pv_generation_components:
        system_id = non_pv['generation_system_id']
        name = non_pv['name']
        system_type = non_pv['system_type']
        model_name = non_pv['model_name']
        manufacturer = non_pv['manufacturer']
        fuel_type = non_pv['fuel_type']
        distribution_systems = non_pv['distribution_systems']
        energy_storage_systems = None
        if non_pv['energy_storage_systems'] is not None:
          storage_component = non_pv['energy_storage_systems']['storage_id']
          storage_systems = self._search_storage_equipment(self._load_storage_components(), storage_component)
          energy_storage_systems = storage_systems
        nominal_heat_output = non_pv['nominal_heat_output']
        maximum_heat_output = non_pv['maximum_heat_output']
        minimum_heat_output = non_pv['minimum_heat_output']
        source_medium = non_pv['source_medium']
        supply_medium = non_pv['supply_medium']
        heat_efficiency = non_pv['heat_efficiency']
        nominal_cooling_output = non_pv['nominal_cooling_output']
        maximum_cooling_output = non_pv['maximum_cooling_output']
        minimum_cooling_output = non_pv['minimum_cooling_output']
        cooling_efficiency = non_pv['cooling_efficiency']
        electricity_efficiency = non_pv['electricity_efficiency']
        source_temperature = non_pv['source_temperature']
        source_mass_flow = non_pv['source_mass_flow']
        nominal_electricity_output = non_pv['nominal_electricity_output']
        maximum_heat_supply_temperature = non_pv['maximum_heat_supply_temperature']
        minimum_heat_supply_temperature = non_pv['minimum_heat_supply_temperature']
        maximum_cooling_supply_temperature = non_pv['maximum_cooling_supply_temperature']
        minimum_cooling_supply_temperature = non_pv['minimum_cooling_supply_temperature']
        heat_output_curve = None
        heat_fuel_consumption_curve = None
        heat_efficiency_curve = None
        cooling_output_curve = None
        cooling_fuel_consumption_curve = None
        cooling_efficiency_curve = None
        if non_pv['heat_output_curve'] is not None:
          curve_type = non_pv['heat_output_curve']['curve_type']
          dependant_variable = non_pv['heat_output_curve']['dependant_variable']
          parameters = non_pv['heat_output_curve']['parameters']
          coefficients = list(non_pv['heat_output_curve']['coefficients'].values())
          heat_output_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        if non_pv['heat_fuel_consumption_curve'] is not None:
          curve_type = non_pv['heat_fuel_consumption_curve']['curve_type']
          dependant_variable = non_pv['heat_fuel_consumption_curve']['dependant_variable']
          parameters = non_pv['heat_fuel_consumption_curve']['parameters']
          coefficients = list(non_pv['heat_fuel_consumption_curve']['coefficients'].values())
          heat_fuel_consumption_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        if non_pv['heat_efficiency_curve'] is not None:
          curve_type = non_pv['heat_efficiency_curve']['curve_type']
          dependant_variable = non_pv['heat_efficiency_curve']['dependant_variable']
          parameters = non_pv['heat_efficiency_curve']['parameters']
          coefficients = list(non_pv['heat_efficiency_curve']['coefficients'].values())
          heat_efficiency_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        if non_pv['cooling_output_curve'] is not None:
          curve_type = non_pv['cooling_output_curve']['curve_type']
          dependant_variable = non_pv['cooling_output_curve']['dependant_variable']
          parameters = non_pv['cooling_output_curve']['parameters']
          coefficients = list(non_pv['cooling_output_curve']['coefficients'].values())
          cooling_output_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        if non_pv['cooling_fuel_consumption_curve'] is not None:
          curve_type = non_pv['cooling_fuel_consumption_curve']['curve_type']
          dependant_variable = non_pv['cooling_fuel_consumption_curve']['dependant_variable']
          parameters = non_pv['cooling_fuel_consumption_curve']['parameters']
          coefficients = list(non_pv['cooling_fuel_consumption_curve']['coefficients'].values())
          cooling_fuel_consumption_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        if non_pv['cooling_efficiency_curve'] is not None:
          curve_type = non_pv['cooling_efficiency_curve']['curve_type']
          dependant_variable = non_pv['cooling_efficiency_curve']['dependant_variable']
          parameters = non_pv['cooling_efficiency_curve']['parameters']
          coefficients = list(non_pv['cooling_efficiency_curve']['coefficients'].values())
          cooling_efficiency_curve = PerformanceCurves(curve_type, dependant_variable, parameters, coefficients)
        dhw = None
        if non_pv['domestic_hot_water'] is not None:
          if non_pv['domestic_hot_water'] == 'True':
            dhw = True
          else:
            dhw = False

        reversible = None
        if non_pv['reversible'] is not None:
          if non_pv['reversible'] == 'True':
            reversible = True
          else:
            reversible = False

        dual_supply = None
        if non_pv['simultaneous_heat_cold'] is not None:
          if non_pv['simultaneous_heat_cold'] == 'True':
            dual_supply = True
          else:
            dual_supply = False
        non_pv_component = NonPvGenerationSystem(system_id=system_id,
                                                 name=name,
                                                 system_type=system_type,
                                                 model_name=model_name,
                                                 manufacturer=manufacturer,
                                                 fuel_type=fuel_type,
                                                 nominal_heat_output=nominal_heat_output,
                                                 maximum_heat_output=maximum_heat_output,
                                                 minimum_heat_output=minimum_heat_output,
                                                 source_medium=source_medium,
                                                 supply_medium=supply_medium,
                                                 heat_efficiency=heat_efficiency,
                                                 nominal_cooling_output=nominal_cooling_output,
                                                 maximum_cooling_output=maximum_cooling_output,
                                                 minimum_cooling_output=minimum_cooling_output,
                                                 cooling_efficiency=cooling_efficiency,
                                                 electricity_efficiency=electricity_efficiency,
                                                 source_temperature=source_temperature,
                                                 source_mass_flow=source_mass_flow,
                                                 nominal_electricity_output=nominal_electricity_output,
                                                 maximum_heat_supply_temperature=maximum_heat_supply_temperature,
                                                 minimum_heat_supply_temperature=minimum_heat_supply_temperature,
                                                 maximum_cooling_supply_temperature=maximum_cooling_supply_temperature,
                                                 minimum_cooling_supply_temperature=minimum_cooling_supply_temperature,
                                                 heat_output_curve=heat_output_curve,
                                                 heat_fuel_consumption_curve=heat_fuel_consumption_curve,
                                                 heat_efficiency_curve=heat_efficiency_curve,
                                                 cooling_output_curve=cooling_output_curve,
                                                 cooling_fuel_consumption_curve=cooling_fuel_consumption_curve,
                                                 cooling_efficiency_curve=cooling_efficiency_curve,
                                                 distribution_systems=distribution_systems,
                                                 energy_storage_systems=energy_storage_systems,
                                                 domestic_hot_water=dhw,
                                                 reversible=reversible,
                                                 simultaneous_heat_cold=dual_supply)
        generation_components.append(non_pv_component)
    pv_generation_components = self._archetypes['EnergySystemCatalog']['energy_generation_components'][
      'pv_generation_component']
    if pv_generation_components is not None:
      for pv in pv_generation_components:
        system_id = pv['generation_system_id']
        name = pv['name']
        system_type = pv['system_type']
        model_name = pv['model_name']
        manufacturer = pv['manufacturer']
        electricity_efficiency = pv['electricity_efficiency']
        nominal_electricity_output = pv['nominal_electricity_output']
        nominal_ambient_temperature = pv['nominal_ambient_temperature']
        nominal_cell_temperature = pv['nominal_cell_temperature']
        nominal_radiation = pv['nominal_radiation']
        standard_test_condition_cell_temperature = pv['standard_test_condition_cell_temperature']
        standard_test_condition_maximum_power = pv['standard_test_condition_maximum_power']
        standard_test_condition_radiation = pv['standard_test_condition_radiation']
        cell_temperature_coefficient = pv['cell_temperature_coefficient']
        width = pv['width']
        height = pv['height']
        distribution_systems = pv['distribution_systems']
        energy_storage_systems = None
        if pv['energy_storage_systems'] is not None:
          storage_component = pv['energy_storage_systems']['storage_id']
          storage_systems = self._search_storage_equipment(self._load_storage_components(), storage_component)
          energy_storage_systems = storage_systems
        pv_component = PvGenerationSystem(system_id=system_id,
                                          name=name,
                                          system_type=system_type,
                                          model_name=model_name,
                                          manufacturer=manufacturer,
                                          electricity_efficiency=electricity_efficiency,
                                          nominal_electricity_output=nominal_electricity_output,
                                          nominal_ambient_temperature=nominal_ambient_temperature,
                                          nominal_cell_temperature=nominal_cell_temperature,
                                          nominal_radiation=nominal_radiation,
                                          standard_test_condition_cell_temperature=
                                          standard_test_condition_cell_temperature,
                                          standard_test_condition_maximum_power=standard_test_condition_maximum_power,
                                          standard_test_condition_radiation=standard_test_condition_radiation,
                                          cell_temperature_coefficient=cell_temperature_coefficient,
                                          width=width,
                                          height=height,
                                          distribution_systems=distribution_systems,
                                          energy_storage_systems=energy_storage_systems)
        generation_components.append(pv_component)

    return generation_components

  def _load_distribution_equipments(self):
    _equipments = []
    distribution_systems = self._archetypes['EnergySystemCatalog']['distribution_systems']['distribution_system']
    if distribution_systems is not None:
      for distribution_system in distribution_systems:
        system_id = None
        model_name = None
        system_type = None
        supply_temperature = None
        distribution_consumption_fix_flow = None
        distribution_consumption_variable_flow = None
        heat_losses = None
        generation_systems = None
        energy_storage_systems = None
        emission_systems = None
        distribution_equipment = DistributionSystem(system_id=system_id,
                                                    model_name=model_name,
                                                    system_type=system_type,
                                                    supply_temperature=supply_temperature,
                                                    distribution_consumption_fix_flow=distribution_consumption_fix_flow,
                                                    distribution_consumption_variable_flow=
                                                    distribution_consumption_variable_flow,
                                                    heat_losses=heat_losses,
                                                    generation_systems=generation_systems,
                                                    energy_storage_systems=energy_storage_systems,
                                                    emission_systems=emission_systems
                                                    )
        _equipments.append(distribution_equipment)
    return _equipments

  def _load_emission_equipments(self):
    _equipments = []
    dissipation_systems = self._archetypes['EnergySystemCatalog']['dissipation_systems']['dissipation_system']
    if dissipation_systems is not None:
      for dissipation_system in dissipation_systems:
        system_id = None
        model_name = None
        system_type = None
        parasitic_energy_consumption = 0
        emission_system = EmissionSystem(system_id=system_id,
                                         model_name=model_name,
                                         system_type=system_type,
                                         parasitic_energy_consumption=parasitic_energy_consumption)
        _equipments.append(emission_system)
    return _equipments

  def _load_storage_components(self):
    storage_components = []
    thermal_storages = self._archetypes['EnergySystemCatalog']['energy_storage_components']['thermalStorages']
    template_storages = self._archetypes['EnergySystemCatalog']['energy_storage_components']['templateStorages']
    for tes in thermal_storages:
      storage_id = tes['storage_id']
      type_energy_stored = tes['type_energy_stored']
      model_name = tes['model_name']
      manufacturer = tes['manufacturer']
      storage_type = tes['storage_type']
      volume = tes['physical_characteristics']['volume']
      height = tes['physical_characteristics']['height']
      maximum_operating_temperature = tes['maximum_operating_temperature']
      materials = self._load_materials()
      insulation_material_id = tes['insulation']['material_id']
      insulation_material = self._search_material(materials, insulation_material_id)
      material_id = tes['physical_characteristics']['material_id']
      tank_material = self._search_material(materials, material_id)
      thickness = float(tes['insulation']['insulationThickness']) / 100  # from cm to m
      insulation_layer = Layer(None, 'insulation', insulation_material, thickness)
      thickness = float(tes['physical_characteristics']['tankThickness']) / 100  # from cm to m
      tank_layer = Layer(None, 'tank', tank_material, thickness)
      media = self._load_media()
      media_id = tes['storage_medium']['medium_id']
      medium = self._search_media(media, media_id)
      layers = [insulation_layer, tank_layer]
      nominal_capacity = tes['nominal_capacity']
      losses_ratio = tes['losses_ratio']
      heating_coil_capacity = tes['heating_coil_capacity']
      storage_component = ThermalStorageSystem(storage_id=storage_id,
                                               model_name=model_name,
                                               type_energy_stored=type_energy_stored,
                                               manufacturer=manufacturer,
                                               storage_type=storage_type,
                                               nominal_capacity=nominal_capacity,
                                               losses_ratio=losses_ratio,
                                               volume=volume,
                                               height=height,
                                               layers=layers,
                                               maximum_operating_temperature=maximum_operating_temperature,
                                               storage_medium=medium,
                                               heating_coil_capacity=heating_coil_capacity)
      storage_components.append(storage_component)

    for template in template_storages:
      storage_id = template['storage_id']
      storage_type = template['storage_type']
      type_energy_stored = template['type_energy_stored']
      maximum_operating_temperature = template['maximum_operating_temperature']
      height = float(template['physical_characteristics']['height'])
      materials = self._load_materials()
      insulation_material_id = template['insulation']['material_id']
      insulation_material = self._search_material(materials, insulation_material_id)
      material_id = template['physical_characteristics']['material_id']
      tank_material = self._search_material(materials, material_id)
      thickness = float(template['insulation']['insulationThickness']) / 100  # from cm to m
      insulation_layer = Layer(None, 'insulation', insulation_material, thickness)
      thickness = float(template['physical_characteristics']['tankThickness']) / 100  # from cm to m
      tank_layer = Layer(None, 'tank', tank_material, thickness)
      layers = [insulation_layer, tank_layer]
      media = self._load_media()
      media_id = template['storage_medium']['medium_id']
      medium = self._search_media(media, media_id)
      model_name = template['model_name']
      manufacturer = template['manufacturer']
      nominal_capacity = template['nominal_capacity']
      losses_ratio = template['losses_ratio']
      volume = template['physical_characteristics']['volume']
      heating_coil_capacity = template['heating_coil_capacity']
      storage_component = ThermalStorageSystem(storage_id=storage_id,
                                               model_name=model_name,
                                               type_energy_stored=type_energy_stored,
                                               manufacturer=manufacturer,
                                               storage_type=storage_type,
                                               nominal_capacity=nominal_capacity,
                                               losses_ratio=losses_ratio,
                                               volume=volume,
                                               height=height,
                                               layers=layers,
                                               maximum_operating_temperature=maximum_operating_temperature,
                                               storage_medium=medium,
                                               heating_coil_capacity=heating_coil_capacity)
      storage_components.append(storage_component)
    return storage_components

  def _load_systems(self):
    base_path = Path(Path(__file__).parent.parent.parent / 'data/energy_systems')
    _catalog_systems = []
    systems = self._archetypes['EnergySystemCatalog']['systems']['system']
    for system in systems:
      system_id = system['id']
      name = system['name']
      demands = system['demands']['demand']
      generation_components = system['components']['generation_id']
      generation_systems = self._search_generation_equipment(self._load_generation_components(), generation_components)
      configuration_schema = Path(base_path / system['schema'])
      energy_system = System(system_id=system_id,
                             name=name,
                             demand_types=demands,
                             generation_systems=generation_systems,
                             distribution_systems=None,
                             configuration_schema=configuration_schema)
      _catalog_systems.append(energy_system)
    return _catalog_systems

  def _load_archetypes(self):
    _system_archetypes = []
    system_clusters = self._archetypes['EnergySystemCatalog']['system_archetypes']['system_archetype']
    for system_cluster in system_clusters:
      archetype_id = system_cluster['@cluster_id']
      name = system_cluster['name']
      systems = system_cluster['systems']['system_id']
      integer_system_ids = [int(item) for item in systems]
      _systems = []
      for system_archetype in self._systems:
        if int(system_archetype.id) in integer_system_ids:
          _systems.append(system_archetype)
      _system_archetypes.append(Archetype(archetype_cluster_id=archetype_id, name=name, systems=_systems))
    return _system_archetypes

  def _load_materials(self):
    materials = []
    _materials = self._archetypes['EnergySystemCatalog']['materials']['material']
    for _material in _materials:
      material_id = _material['material_id']
      name = _material['name']
      conductivity = _material['conductivity']
      solar_absorptance = _material['solar_absorptance']
      thermal_absorptance = _material['thermal_absorptance']
      density = _material['density']
      specific_heat = _material['specific_heat']
      no_mass = _material['no_mass']
      visible_absorptance = _material['visible_absorptance']
      thermal_resistance = _material['thermal_resistance']

      material = Material(material_id,
                          name,
                          solar_absorptance=solar_absorptance,
                          thermal_absorptance=thermal_absorptance,
                          density=density,
                          conductivity=conductivity,
                          thermal_resistance=thermal_resistance,
                          visible_absorptance=visible_absorptance,
                          no_mass=no_mass,
                          specific_heat=specific_heat)
      materials.append(material)
    return materials

  @staticmethod
  def _search_material(materials, material_id):
    _material = None
    for material in materials:
      if int(material.id) == int(material_id):
        _material = material
        break
    if _material is None:
      raise ValueError(f'Material with the id = [{material_id}] not found in catalog ')
    return _material

  def _load_media(self):
    media = []
    _media = [self._archetypes['EnergySystemCatalog']['media']['medium']]
    for _medium in _media:
      medium_id = _medium['medium_id']
      density = _medium['density']
      name = _medium['name']
      conductivity = _medium['conductivity']
      solar_absorptance = _medium['solar_absorptance']
      thermal_absorptance = _medium['thermal_absorptance']
      specific_heat = _medium['specific_heat']
      no_mass = _medium['no_mass']
      visible_absorptance = _medium['visible_absorptance']
      thermal_resistance = _medium['thermal_resistance']
      medium = Material(material_id=medium_id,
                        name=name,
                        solar_absorptance=solar_absorptance,
                        thermal_absorptance=thermal_absorptance,
                        visible_absorptance=visible_absorptance,
                        no_mass=no_mass,
                        thermal_resistance=thermal_resistance,
                        conductivity=conductivity,
                        density=density,
                        specific_heat=specific_heat)
      media.append(medium)
    return media

  @staticmethod
  def _search_media(media, medium_id):
    _medium = None
    for medium in media:
      if int(medium.id) == int(medium_id):
        _medium = medium
        break
    if _medium is None:
      raise ValueError(f'media with the id = [{medium_id}] not found in catalog ')
    return _medium

  @staticmethod
  def _search_generation_equipment(generation_systems, generation_id):
    _generation_systems = []

    if isinstance(generation_id, list):
      integer_ids = [int(item) for item in generation_id]
      for generation in generation_systems:
        if int(generation.id) in integer_ids:
          _generation_systems.append(generation)
    else:
      integer_id = int(generation_id)
      for generation in generation_systems:
        if int(generation.id) == integer_id:
          _generation_systems.append(generation)

    if len(_generation_systems) == 0:
      _generation_systems = None
      raise ValueError(f'The system with the following id is not found in catalog [{generation_id}]')
    return _generation_systems

  @staticmethod
  def _search_storage_equipment(storage_systems, storage_id):
    _storage_systems = []
    for storage in storage_systems:
      if storage.id in storage_id:
        _storage_systems.append(storage)
    if len(_storage_systems) == 0:
      _storage_systems = None
      raise ValueError(f'The system with the following id is not found in catalog [{storage_id}]')
    return _storage_systems

  def names(self, category=None):
    """
    Get the catalog elements names
    :parm: optional category filter
    """
    if category is None:
      _names = {'archetypes': [], 'systems': [], 'generation_equipments': [], 'storage_equipments': []}
      for archetype in self._content.archetypes:
        _names['archetypes'].append(archetype.name)
      for system in self._content.systems:
        _names['systems'].append(system.name)
      for equipment in self._content.generation_equipments:
        _names['generation_equipments'].append(equipment.name)
    else:
      _names = {category: []}
      if category.lower() == 'archetypes':
        for archetype in self._content.archetypes:
          _names[category].append(archetype.name)
      elif category.lower() == 'systems':
        for system in self._content.systems:
          _names[category].append(system.name)
      elif category.lower() == 'generation_equipments':
        for system in self._content.generation_equipments:
          _names[category].append(system.name)
      else:
        raise ValueError(f'Unknown category [{category}]')
    return _names

  def entries(self, category=None):
    """
    Get the catalog elements
    :parm: optional category filter
    """
    if category is None:
      return self._content
    if category.lower() == 'archetypes':
      return self._content.archetypes
    if category.lower() == 'systems':
      return self._content.systems
    if category.lower() == 'generation_equipments':
      return self._content.generation_equipments
    raise ValueError(f'Unknown category [{category}]')

  def get_entry(self, name):
    """
    Get one catalog element by names
    :parm: entry name
    """
    for entry in self._content.archetypes:
      if entry.name.lower() == name.lower():
        return entry
    for entry in self._content.systems:
      if entry.name.lower() == name.lower():
        return entry
    for entry in self._content.generation_equipments:
      if entry.name.lower() == name.lower():
        return entry
    raise IndexError(f"{name} doesn't exists in the catalog")
